@@ -24,7 +24,7 @@
 #include "bfd.h"
 #include "libbfd.h"
 #include "elf-bfd.h"
-#include "dwarf2.h"
+#include "elf/dwarf2.h"
 
 #define EH_FRAME_HDR_SIZE 8
 
@@ -537,21 +537,18 @@ _bfd_elf_parse_eh_frame (bfd *abfd, struct bfd_link_info *info,
   local_cies = bfd_zmalloc (num_cies * sizeof (*local_cies));
   REQUIRE (local_cies);
 
-  /* FIXME: octets_per_byte.  */
 #define ENSURE_NO_RELOCS(buf)				\
   REQUIRE (!(cookie->rel < cookie->relend		\
 	     && (cookie->rel->r_offset			\
 		 < (bfd_size_type) ((buf) - ehbuf))	\
 	     && cookie->rel->r_info != 0))
 
-  /* FIXME: octets_per_byte.  */
 #define SKIP_RELOCS(buf)				\
   while (cookie->rel < cookie->relend			\
 	 && (cookie->rel->r_offset			\
 	     < (bfd_size_type) ((buf) - ehbuf)))	\
     cookie->rel++
 
-  /* FIXME: octets_per_byte.  */
 #define GET_RELOC(buf)					\
   ((cookie->rel < cookie->relend			\
     && (cookie->rel->r_offset				\
@@ -914,7 +911,6 @@ mark_entry (struct bfd_link_info *info, asection *sec,
 	    struct eh_cie_fde *ent, elf_gc_mark_hook_fn gc_mark_hook,
 	    struct elf_reloc_cookie *cookie)
 {
-  /* FIXME: octets_per_byte.  */
   for (cookie->rel = cookie->rels + ent->reloc_index;
        cookie->rel < cookie->relend
 	 && cookie->rel->r_offset < ent->offset + ent->size;
@@ -1103,7 +1099,6 @@ _bfd_elf_discard_section_eh_frame
     else if (!ent->cie)
       {
 	cookie->rel = cookie->rels + ent->reloc_index;
-	/* FIXME: octets_per_byte.  */
 	BFD_ASSERT (cookie->rel < cookie->relend
 		    && cookie->rel->r_offset == ent->offset + 8);
 	if (!(*reloc_symbol_deleted_p) (ent->offset + 8, cookie))
@@ -1328,7 +1323,6 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
   struct eh_cie_fde *ent;
 
   if (sec->sec_info_type != ELF_INFO_TYPE_EH_FRAME)
-    /* FIXME: octets_per_byte.  */
     return bfd_set_section_contents (abfd, sec->output_section, contents,
 				     sec->output_offset, sec->size);
 
@@ -1617,7 +1611,6 @@ _bfd_elf_write_section_eh_frame (bfd *abfd,
   if ((sec->size % ptr_size) != 0)
     abort ();
 
-  /* FIXME: octets_per_byte.  */
   return bfd_set_section_contents (abfd, sec->output_section,
 				   contents, (file_ptr) sec->output_offset,
 				   sec->size);
@@ -1729,7 +1722,6 @@ _bfd_elf_write_section_eh_frame_hdr (bfd *abfd, struct bfd_link_info *info)
 	}
     }
 
-  /* FIXME: octets_per_byte.  */
   retval = bfd_set_section_contents (abfd, sec->output_section,
 				     contents, (file_ptr) sec->output_offset,
 				     sec->size);
