@@ -1,6 +1,6 @@
 /* ELF64/HPPA support
 
-   Copyright 1999, 2000, 2002, 2007 Free Software Foundation, Inc.
+   Copyright 1999, 2000, 2002, 2007, 2009 Free Software Foundation, Inc.
 
    This file is part of BFD, the Binary File Descriptor library.
 
@@ -26,11 +26,11 @@
 #include "libhppa.h"
 #include "elf/hppa.h"
 
-elf_hppa_reloc_type elf64_hppa_reloc_final_type
-  PARAMS ((bfd *, elf_hppa_reloc_type, int, unsigned int));
+extern elf_hppa_reloc_type elf64_hppa_reloc_final_type
+  (bfd *, elf_hppa_reloc_type, int, unsigned int);
 
 extern elf_hppa_reloc_type ** _bfd_elf64_hppa_gen_reloc_type
-  PARAMS ((bfd *, elf_hppa_reloc_type, int, unsigned int, int, asymbol *));
+  (bfd *, elf_hppa_reloc_type, int, unsigned int, int, asymbol *);
 
 /* Define groups of basic relocations.  FIXME:  These should
    be the only basic relocations created by GAS.  The rest
