@@ -1,6 +1,6 @@
 /* ELF executable support for BFD.
    Copyright 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
-   2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
+   2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    Free Software Foundation, Inc.
 
    Written by Fred Fish @ Cygnus Support, from information published
@@ -194,6 +194,10 @@ elf_swap_symbol_in (bfd *abfd,
       if (shndx == NULL)
 	return FALSE;
       dst->st_shndx = H_GET_32 (abfd, shndx->est_shndx);
+      /* Handle gap in section indices.  */
+      if (bfd_has_gap_in_elf_shndx (abfd)
+	  && dst->st_shndx > (SHN_HIRESERVE & 0xffff))
+	dst->st_shndx -= ELF_SECTION_HEADER_INDEX_GAP;
     }
   else if (dst->st_shndx >= (SHN_LORESERVE & 0xffff))
     dst->st_shndx += SHN_LORESERVE - (SHN_LORESERVE & 0xffff);
@@ -688,8 +692,14 @@ elf_object_p (bfd *abfd)
       if (i_ehdrp->e_shnum == SHN_UNDEF)
 	{
 	  i_ehdrp->e_shnum = i_shdr.sh_size;
-	  if (i_ehdrp->e_shnum != i_shdr.sh_size
-	      || i_ehdrp->e_shnum == 0)
+	  if (i_ehdrp->e_shnum >= SHN_LORESERVE)
+	    {
+	      _bfd_error_handler (_("%B: too many sections: %u"),
+				  abfd, i_ehdrp->e_shnum);
+	      abort ();
+	    }
+	  else if (i_ehdrp->e_shnum != i_shdr.sh_size
+		   || i_ehdrp->e_shnum  == 0)
 	    goto got_wrong_format_error;
 	}
 
@@ -758,15 +768,29 @@ elf_object_p (bfd *abfd)
 	    goto got_no_match;
 	  elf_swap_shdr_in (abfd, &x_shdr, i_shdrp + shindex);
 
-	  /* Sanity check sh_link and sh_info.  */
+	  /* Sanity check sh_link and sh_info.  The older linker before
+	     fix for PR ld/5900 creates a gap in section indices.  Set
+	     bfd_has_gap_in_elf_shndx to 1 if there is a gap.  */
 	  if (i_shdrp[shindex].sh_link >= num_sec)
-	    goto got_wrong_format_error;
+	    {
+	      if (i_shdrp[shindex].sh_link <= (SHN_HIRESERVE & 0xffff)
+		  || ((i_shdrp[shindex].sh_link
+		       - ELF_SECTION_HEADER_INDEX_GAP) >= num_sec))
+		goto got_wrong_format_error;
+	      bfd_has_gap_in_elf_shndx (abfd) = 1;
+	    }
 
 	  if (((i_shdrp[shindex].sh_flags & SHF_INFO_LINK)
 	       || i_shdrp[shindex].sh_type == SHT_RELA
 	       || i_shdrp[shindex].sh_type == SHT_REL)
 	      && i_shdrp[shindex].sh_info >= num_sec)
-	    goto got_wrong_format_error;
+	    {
+	      if (i_shdrp[shindex].sh_info <= (SHN_HIRESERVE & 0xffff)
+		  || ((i_shdrp[shindex].sh_info
+		       - ELF_SECTION_HEADER_INDEX_GAP) >= num_sec))
+		goto got_wrong_format_error;
+	      bfd_has_gap_in_elf_shndx (abfd) = 1;
+	    }
 
 	  /* If the section is loaded, but not page aligned, clear
 	     D_PAGED.  */
@@ -778,6 +802,28 @@ elf_object_p (bfd *abfd)
 		  != 0))
 	    abfd->flags &= ~D_PAGED;
 	}
+
+      /* To support files generated by the older linker, we adjust
+	 section indices if there is a gap.  */
+      if (bfd_has_gap_in_elf_shndx (abfd))
+	{
+	  BFD_ASSERT (i_ehdrp->e_shstrndx == i_shdrp->sh_link);
+
+	  for (; num_sec > 0; num_sec--, i_shdrp++)
+	    {
+	      if (i_shdrp->sh_link > (SHN_HIRESERVE & 0xffff))
+		i_shdrp->sh_link -= ELF_SECTION_HEADER_INDEX_GAP;
+
+	      if (i_shdrp->sh_info > (SHN_HIRESERVE & 0xffff)
+		  && ((i_shdrp->sh_flags & SHF_INFO_LINK)
+		      || i_shdrp->sh_type == SHT_RELA
+		      || i_shdrp->sh_type == SHT_REL))
+		i_shdrp->sh_info -= ELF_SECTION_HEADER_INDEX_GAP;
+	    }
+
+	  if (i_ehdrp->e_shstrndx > (SHN_HIRESERVE & 0xffff))
+	    i_ehdrp->e_shstrndx -= ELF_SECTION_HEADER_INDEX_GAP;
+	}
     }
 
   /* A further sanity check.  */
@@ -792,7 +838,8 @@ elf_object_p (bfd *abfd)
 	     So we are kind, and reset the string index value to 0
 	     so that at least some processing can be done.  */
 	  i_ehdrp->e_shstrndx = SHN_UNDEF;
-	  _bfd_error_handler (_("warning: %s has a corrupt string table index - ignoring"), abfd->filename);
+	  _bfd_error_handler (_("warning: %B has a corrupt string table index - ignoring"),
+			      abfd);
 	}
     }
   else if (i_ehdrp->e_shstrndx != SHN_UNDEF)
@@ -1282,9 +1329,6 @@ elf_slurp_symbol_table (bfd *abfd, asymbol **symptrs, bfd_boolean dynamic)
 	    case STB_WEAK:
 	      sym->symbol.flags |= BSF_WEAK;
 	      break;
-	    case STB_GNU_UNIQUE:
-	      sym->symbol.flags |= BSF_GNU_UNIQUE;
-	      break;
 	    }
 
 	  switch (ELF_ST_TYPE (isym->st_info))
