@@ -1,6 +1,6 @@
 /* BFD backend for Extended Tektronix Hex Format  objects.
    Copyright 1992, 1993, 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002,
-   2003, 2004, 2005, 2006, 2007, 2009 Free Software Foundation, Inc.
+   2003, 2004, 2007 Free Software Foundation, Inc.
    Written by Steve Chamberlain of Cygnus Support <sac@cygnus.com>.
 
    This file is part of BFD, the Binary File Descriptor library.
@@ -415,33 +415,31 @@ first_phase (bfd *abfd, int type, char *src)
 	      /* Symbols, add to section.  */
 	      {
 		bfd_size_type amt = sizeof (tekhex_symbol_type);
-		tekhex_symbol_type *new_symbol = (tekhex_symbol_type *)
-                    bfd_alloc (abfd, amt);
+		tekhex_symbol_type *new = bfd_alloc (abfd, amt);
 		char stype = (*src);
 
-		if (!new_symbol)
+		if (!new)
 		  return FALSE;
-		new_symbol->symbol.the_bfd = abfd;
+		new->symbol.the_bfd = abfd;
 		src++;
 		abfd->symcount++;
 		abfd->flags |= HAS_SYMS;
-		new_symbol->prev = abfd->tdata.tekhex_data->symbols;
-		abfd->tdata.tekhex_data->symbols = new_symbol;
+		new->prev = abfd->tdata.tekhex_data->symbols;
+		abfd->tdata.tekhex_data->symbols = new;
 		if (!getsym (sym, &src, &len))
 		  return FALSE;
-		new_symbol->symbol.name = (const char *)
-                    bfd_alloc (abfd, (bfd_size_type) len + 1);
-		if (!new_symbol->symbol.name)
+		new->symbol.name = bfd_alloc (abfd, (bfd_size_type) len + 1);
+		if (!new->symbol.name)
 		  return FALSE;
-		memcpy ((char *) (new_symbol->symbol.name), sym, len + 1);
-		new_symbol->symbol.section = section;
+		memcpy ((char *) (new->symbol.name), sym, len + 1);
+		new->symbol.section = section;
 		if (stype <= '4')
-		  new_symbol->symbol.flags = (BSF_GLOBAL | BSF_EXPORT);
+		  new->symbol.flags = (BSF_GLOBAL | BSF_EXPORT);
 		else
-		  new_symbol->symbol.flags = BSF_LOCAL;
+		  new->symbol.flags = BSF_LOCAL;
 		if (!getvalue (&src, &val))
 		  return FALSE;
-		new_symbol->symbol.value = val - section->vma;
+		new->symbol.value = val - section->vma;
 		break;
 	      }
 	    default:
@@ -882,14 +880,13 @@ static asymbol *
 tekhex_make_empty_symbol (bfd *abfd)
 {
   bfd_size_type amt = sizeof (struct tekhex_symbol_struct);
-  tekhex_symbol_type *new_symbol = (tekhex_symbol_type *) bfd_zalloc (abfd,
-                                                                      amt);
+  tekhex_symbol_type *new = bfd_zalloc (abfd, amt);
 
-  if (!new_symbol)
+  if (!new)
     return NULL;
-  new_symbol->symbol.the_bfd = abfd;
-  new_symbol->prev =  NULL;
-  return &(new_symbol->symbol);
+  new->symbol.the_bfd = abfd;
+  new->prev =  NULL;
+  return &(new->symbol);
 }
 
 static void
