@@ -1,6 +1,6 @@
 /* nm.c -- Describe symbol table of a rel file.
    Copyright 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
-   2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009
+   2001, 2002, 2003, 2004, 2005, 2007, 2009
    Free Software Foundation, Inc.
 
    This file is part of GNU Binutils.
@@ -240,13 +240,11 @@ usage (FILE *stream, int status)
   -o                     Same as -A\n\
   -p, --no-sort          Do not sort the symbols\n\
   -P, --portability      Same as --format=posix\n\
-  -r, --reverse-sort     Reverse the sense of the sort\n"));
+  -r, --reverse-sort     Reverse the sense of the sort\n"
 #if BFD_SUPPORTS_PLUGINS
-  fprintf (stream, _("\
-      --plugin NAME      Load the specified plugin\n"));
+"      --plugin NAME      Load the specified plugin\n"
 #endif
-  fprintf (stream, _("\
-  -S, --print-size       Print size of defined symbols\n		\
+"  -S, --print-size       Print size of defined symbols\n\
   -s, --print-armap      Include index for symbols from archive members\n\
       --size-sort        Sort symbols by size\n\
       --special-syms     Include special symbols in the output\n\
