@@ -104,7 +104,6 @@ static bfd_boolean disassemble_all;	/* -D */
 static int disassemble_zeroes;		/* --disassemble-zeroes */
 static bfd_boolean formats_info;	/* -i */
 static int wide_output;			/* -w */
-static int insn_width;			/* --insn-width */
 static bfd_vma start_address = (bfd_vma) -1; /* --start-address */
 static bfd_vma stop_address = (bfd_vma) -1;  /* --stop-address */
 static int dump_debugging;		/* --debugging */
@@ -236,7 +235,6 @@ usage (FILE *stream, int status)
       --stop-address=ADDR        Only process data whose address is <= ADDR\n\
       --prefix-addresses         Print complete address alongside disassembly\n\
       --[no-]show-raw-insn       Display hex alongside symbolic disassembly\n\
-      --insn-width=WIDTH         Display WIDTH bytes on a signle line for -d\n\
       --adjust-vma=OFFSET        Add OFFSET to all displayed section addresses\n\
       --special-syms             Include special symbols in symbol dumps\n\
       --prefix=PREFIX            Add PREFIX to absolute paths for -S\n\
@@ -261,7 +259,6 @@ enum option_values
     OPTION_DWARF,
     OPTION_PREFIX,
     OPTION_PREFIX_STRIP,
-    OPTION_INSN_WIDTH,
     OPTION_ADJUST_VMA
   };
 
@@ -309,7 +306,6 @@ static struct option long_options[]=
   {"wide", no_argument, NULL, 'w'},
   {"prefix", required_argument, NULL, OPTION_PREFIX},
   {"prefix-strip", required_argument, NULL, OPTION_PREFIX_STRIP},
-  {"insn-width", required_argument, NULL, OPTION_INSN_WIDTH},
   {0, no_argument, 0, 0}
 };
 
@@ -320,23 +316,6 @@ nonfatal (const char *msg)
   exit_status = 1;
 }
 
-/* Returns TRUE if the specified section should be dumped.  */
-
-static bfd_boolean
-process_section_p (asection * section)
-{
-  size_t i;
-
-  if (only == NULL)
-    return TRUE;
-
-  for (i = 0; i < only_used; i++)
-    if (strcmp (only [i], section->name) == 0)
-      return TRUE;
-
-  return FALSE;
-}
-
 static void
 dump_section_header (bfd *abfd, asection *section,
 		     void *ignored ATTRIBUTE_UNUSED)
@@ -349,10 +328,6 @@ dump_section_header (bfd *abfd, asection *section,
   if (section->flags & SEC_LINKER_CREATED)
     return;
 
-  /* PR 10413: Skip sections that we are ignoring.  */
-  if (! process_section_p (section))
-    return;
-
   printf ("%3d %-13s %08lx  ", section->index,
 	  bfd_get_section_name (abfd, section),
 	  (unsigned long) bfd_section_size (abfd, section) / opb);
@@ -1376,6 +1351,24 @@ objdump_sprintf (SFILE *f, const char *format, ...)
   return n;
 }
 
+/* Returns TRUE if the specified section should be dumped.  */
+
+static bfd_boolean
+process_section_p (asection * section)
+{
+  size_t i;
+
+  if (only == NULL)
+    return TRUE;
+
+  for (i = 0; i < only_used; i++)
+    if (strcmp (only [i], section->name) == 0)
+      return TRUE;
+
+  return FALSE;
+}
+
+
 /* The number of zeroes we want to see before we start skipping them.
    The number is arbitrarily chosen.  */
 
@@ -1422,9 +1415,7 @@ disassemble_bytes (struct disassemble_info * info,
   sfile.buffer = xmalloc (sfile.alloc);
   sfile.pos = 0;
   
-  if (insn_width)
-    octets_per_line = insn_width;
-  else if (insns)
+  if (insns)
     octets_per_line = 4;
   else
     octets_per_line = 16;
@@ -1534,9 +1525,7 @@ disassemble_bytes (struct disassemble_info * info,
 	      info->stream = &sfile;
 	      info->bytes_per_line = 0;
 	      info->bytes_per_chunk = 0;
-	      info->flags = disassemble_all ? DISASSEMBLE_DATA : 0;
-	      if (machine)
-		info->flags |= USER_SPECIFIED_MACHINE_TYPE;
+	      info->flags = 0;
 
 	      if (info->disassembler_needs_relocs
 		  && (bfd_get_file_flags (aux->abfd) & EXEC_P) == 0
@@ -1565,7 +1554,7 @@ disassemble_bytes (struct disassemble_info * info,
 		      || (distance_to_rel > 0
 			  && distance_to_rel < (bfd_signed_vma) (previous_octets/ opb)))
 		    {
-		      info->flags |= INSN_HAS_RELOC;
+		      info->flags = INSN_HAS_RELOC;
 		      aux->reloc = **relppp;
 		    }
 		  else
@@ -1575,7 +1564,7 @@ disassemble_bytes (struct disassemble_info * info,
 	      octets = (*disassemble_fn) (section->vma + addr_offset, info);
 	      info->fprintf_func = (fprintf_ftype) fprintf;
 	      info->stream = stdout;
-	      if (insn_width == 0 && info->bytes_per_line != 0)
+	      if (info->bytes_per_line != 0)
 		octets_per_line = info->bytes_per_line;
 	      if (octets < 0)
 		{
@@ -1753,8 +1742,15 @@ disassemble_bytes (struct disassemble_info * info,
 
 	      if (q->addend)
 		{
-		  printf ("+0x");
-		  objdump_print_value (q->addend, info, TRUE);
+		  bfd_signed_vma addend = q->addend;
+		  if (addend < 0)
+		    {
+		      printf ("-0x");
+		      addend = -addend;
+		    }
+		  else
+		    printf ("+0x");
+		  objdump_print_value (addend, info, TRUE);
 		}
 
 	      printf ("\n");
@@ -2245,18 +2241,21 @@ dump_dwarf_section (bfd *abfd, asection *section,
 	&& debug_displays [i].enabled != NULL
 	&& *debug_displays [i].enabled)
       {
-	struct dwarf_section *sec = &debug_displays [i].section;
-
-	if (strcmp (sec->uncompressed_name, match) == 0)
-	  sec->name = sec->uncompressed_name;
-	else
-	  sec->name = sec->compressed_name;
-	if (load_specific_debug_section (i, section, abfd))
+	if (!debug_displays [i].eh_frame)
 	  {
-	    debug_displays [i].display (sec, abfd);
-	    
-	    if (i != info && i != abbrev)
-	      free_debug_section (i);
+	    struct dwarf_section *sec = &debug_displays [i].section;
+
+	    if (strcmp (sec->uncompressed_name, match) == 0)
+	      sec->name = sec->uncompressed_name;
+	    else
+	      sec->name = sec->compressed_name;
+	    if (load_specific_debug_section (i, section, abfd))
+	      {
+		debug_displays [i].display (sec, abfd);
+
+		if (i != info && i != abbrev)
+		  free_debug_section (i);
+	      }
 	  }
 	break;
       }
@@ -2829,8 +2828,15 @@ dump_reloc_set (bfd *abfd, asection *sec, arelent **relpp, long relcount)
 
       if (q->addend)
 	{
-	  printf ("+0x");
-	  bfd_printf_vma (abfd, q->addend);
+	  bfd_signed_vma addend = q->addend;
+	  if (addend < 0)
+	    {
+	      printf ("-0x");
+	      addend = -addend;
+	    }
+	  else
+	    printf ("+0x");
+	  bfd_printf_vma (abfd, addend);
 	}
 
       printf ("\n");
@@ -3260,11 +3266,6 @@ main (int argc, char **argv)
 	  if (prefix_strip < 0)
 	    fatal (_("error: prefix strip must be non-negative"));
 	  break;
-	case OPTION_INSN_WIDTH:
-	  insn_width = strtoul (optarg, NULL, 0);
-	  if (insn_width <= 0)
-	    fatal (_("error: instruction width must be positive"));
-	  break;
 	case 'E':
 	  if (strcmp (optarg, "B") == 0)
 	    endian = BFD_ENDIAN_BIG;
