@@ -1,6 +1,6 @@
 /* coff object file format
    Copyright 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998,
-   1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2009
+   1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007
    Free Software Foundation, Inc.
 
    This file is part of GAS.
@@ -53,8 +53,6 @@ static symbolS *def_symbol_in_progress;
 static const char weak_altprefix[] = ".weak.";
 #endif /* TE_PE */
 
-#include "obj-coff-seh.c"
-
 typedef struct
   {
     unsigned long chunk_size;
@@ -250,7 +248,7 @@ fetch_coff_debug_section (void)
       const asymbol *s;
 
       s = bfd_make_debug_symbol (stdoutput, NULL, 0);
-      gas_assert (s != 0);
+      assert (s != 0);
       debug_section = s->section;
     }
   return debug_section;
@@ -1100,7 +1098,7 @@ weak_altname2name (const char * name)
   char * weak_name;
   char * dot;
 
-  gas_assert (weak_is_altname (name));
+  assert (weak_is_altname (name));
 
   weak_name = xstrdup (name + 6);
   if ((dot = strchr (weak_name, '.')))
@@ -1121,7 +1119,7 @@ weak_uniquify (const char * name)
   if (an_external_name != NULL)
     unique = an_external_name;
 #endif
-  gas_assert (weak_is_altname (name));
+  assert (weak_is_altname (name));
 
   if (strchr (name + sizeof (weak_altprefix), '.'))
     return name;
@@ -1251,8 +1249,8 @@ coff_frob_symbol (symbolS *symp, int *punt)
       symbolS *weakp = symbol_find_noref (weak_altname2name
 					  (S_GET_NAME (symp)), 1);
 
-      gas_assert (weakp);
-      gas_assert (S_GET_NUMBER_AUXILIARY (weakp) == 1);
+      assert (weakp);
+      assert (S_GET_NUMBER_AUXILIARY (weakp) == 1);
 
       if (! S_IS_WEAK (weakp))
 	{
@@ -1334,7 +1332,7 @@ coff_frob_symbol (symbolS *symp, int *punt)
 
       if (!S_IS_DEFINED (symp) && !SF_GET_LOCAL (symp))
 	{
-	  gas_assert (S_GET_VALUE (symp) == 0);
+	  assert (S_GET_VALUE (symp) == 0);
 	  if (S_IS_WEAKREFD (symp))
 	    *punt = 1;
 	  else
@@ -1436,7 +1434,7 @@ coff_frob_symbol (symbolS *symp, int *punt)
   if (next_set_end != NULL)
     {
       if (set_end != NULL)
-	as_warn (_("Warning: internal error: forgetting to set endndx of %s"),
+	as_warn ("Warning: internal error: forgetting to set endndx of %s",
 		 S_GET_NAME (set_end));
       set_end = next_set_end;
     }
@@ -1770,6 +1768,7 @@ coff_frob_section (segT sec)
       SF_SET_STATICS (secsym);
       SA_SET_SCN_SCNLEN (secsym, size);
     }
+
   /* FIXME: These should be in a "stabs.h" file, or maybe as.h.  */
 #ifndef STAB_SECTION_NAME
 #define STAB_SECTION_NAME ".stab"
@@ -1791,7 +1790,7 @@ coff_frob_section (segT sec)
   fragp = seg_info (sec)->frchainP->frch_root;
   while (fragp && fragp->fr_fix == 0)
     fragp = fragp->fr_next;
-  gas_assert (fragp != 0 && fragp->fr_fix >= 12);
+  assert (fragp != 0 && fragp->fr_fix >= 12);
 
   /* Store the values.  */
   p = fragp->fr_literal;
@@ -1877,9 +1876,6 @@ const pseudo_typeS coff_pseudo_table[] =
 #if defined TC_TIC4X
   /* The tic4x uses sdef instead of def.  */
   {"sdef", obj_coff_def, 0},
-#endif
-#if defined(SEH_CMDS)
-  SEH_CMDS
 #endif
   {NULL, NULL, 0}
 };
