@@ -82,7 +82,7 @@ struct fix;
 #define TC_FORCE_RELOCATION(FIX) arm_force_relocation (FIX)
 
 #define md_relax_frag(segment, fragp, stretch) \
-  arm_relax_frag (segment, fragp, stretch)
+  arm_relax_frag(segment, fragp, stretch)
 extern int arm_relax_frag (asection *, struct frag *, long);
 
 #define md_optimize_expr(l,o,r)		arm_optimize_expr (l, o, r)
@@ -123,31 +123,27 @@ bfd_boolean arm_is_eabi (void);
 
 #define ARM_IS_THUMB(s)		(ARM_GET_FLAG (s) & ARM_FLAG_THUMB)
 #define ARM_IS_INTERWORK(s)	(ARM_GET_FLAG (s) & ARM_FLAG_INTERWORK)
-
 #ifdef OBJ_ELF
 
 /* For ELF objects THUMB_IS_FUNC is inferred from
    ARM_IS_THUMB and the function type.  */
-#define THUMB_IS_FUNC(s)					\
-  ((s) != NULL							\
-   && ((arm_is_eabi ()						\
-	&& (ARM_IS_THUMB (s))					\
-	&& (symbol_get_bfdsym (s)->flags & BSF_FUNCTION))	\
-       || (ARM_GET_FLAG (s) & THUMB_FLAG_FUNC)))
+#define THUMB_IS_FUNC(s) \
+  ((arm_is_eabi () \
+    && (ARM_IS_THUMB (s)) \
+    && (symbol_get_bfdsym (s)->flags & BSF_FUNCTION)) \
+   || (ARM_GET_FLAG (s) & THUMB_FLAG_FUNC))
 
-#define ARM_IS_FUNC(s)					\
-  (((s) != NULL						\
-    && arm_is_eabi ()					\
-    && !(ARM_IS_THUMB (s))				\
-    /* && !(THUMB_FLAG_FUNC & ARM_GET_FLAG (s)) \ */	\
+#define ARM_IS_FUNC(s) \
+  ((arm_is_eabi () \
+    && !(ARM_IS_THUMB (s)) \
+    /* && !(THUMB_FLAG_FUNC & ARM_GET_FLAG (s)) \ */ \
     && (symbol_get_bfdsym (s)->flags & BSF_FUNCTION)))
 
 
 #else
-
-#define THUMB_IS_FUNC(s)	((s) && ARM_GET_FLAG (s) & THUMB_FLAG_FUNC)
+#define THUMB_IS_FUNC(s)	(ARM_GET_FLAG (s) & THUMB_FLAG_FUNC)
 #define ARM_IS_FUNC(s)          (!THUMB_IS_FUNC (s) \
-				 && (s) && (symbol_get_bfdsym (s)->flags & BSF_FUNCTION))
+                                && (symbol_get_bfdsym (s)->flags & BSF_FUNCTION))
 #endif
 
 #define ARM_SET_THUMB(s,t)      ((t) ? ARM_SET_FLAG (s, ARM_FLAG_THUMB)     : ARM_RESET_FLAG (s, ARM_FLAG_THUMB))
@@ -155,7 +151,6 @@ bfd_boolean arm_is_eabi (void);
 #define THUMB_SET_FUNC(s,t)     ((t) ? ARM_SET_FLAG (s, THUMB_FLAG_FUNC)    : ARM_RESET_FLAG (s, THUMB_FLAG_FUNC))
 
 void arm_copy_symbol_attributes (symbolS *, symbolS *);
-
 #ifndef TC_COPY_SYMBOL_ATTRIBUTES
 #define TC_COPY_SYMBOL_ATTRIBUTES(DEST, SRC) \
   (arm_copy_symbol_attributes (DEST, SRC))
@@ -193,27 +188,13 @@ void arm_copy_symbol_attributes (symbolS *, symbolS *);
 
 #define TC_CONS_FIX_NEW cons_fix_new_arm
 
-#define MAX_MEM_ALIGNMENT_BYTES    6
-#define MAX_MEM_FOR_RS_ALIGN_CODE ((1 << MAX_MEM_ALIGNMENT_BYTES) - 1)
+#define MAX_MEM_FOR_RS_ALIGN_CODE 31
 
 /* For frags in code sections we need to record whether they contain
    ARM code or THUMB code.  This is that if they have to be aligned,
    they can contain the correct type of no-op instruction.  */
-struct arm_frag_type
-{
-  int thumb_mode;
-#ifdef OBJ_ELF
-  /* If there is a mapping symbol at offset 0 in this frag,
-     it will be saved in FIRST_MAP.  If there are any mapping
-     symbols in this frag, the last one will be saved in
-     LAST_MAP.  */
-  symbolS *first_map, *last_map;
-#endif
-};
-
-#define TC_FRAG_TYPE		struct arm_frag_type
-/* NOTE: max_chars is a local variable from frag_var / frag_variant.  */
-#define TC_FRAG_INIT(fragp)	arm_init_frag (fragp, max_chars)
+#define TC_FRAG_TYPE		int
+#define TC_FRAG_INIT(fragp)	arm_init_frag (fragp)
 #define HANDLE_ALIGN(fragp)	arm_handle_align (fragp)
 
 #define md_do_align(N, FILL, LEN, MAX, LABEL)					\
@@ -231,21 +212,6 @@ struct arm_frag_type
 /* Registers are generally saved at negative offsets to the CFA.  */
 #define DWARF2_CIE_DATA_ALIGNMENT     (-4)
 
-/* State variables for IT block handling.  */
-enum it_state
-{
-  OUTSIDE_IT_BLOCK, MANUAL_IT_BLOCK, AUTOMATIC_IT_BLOCK
-};
-struct current_it
-{
-  int mask;
-  enum it_state state;
-  int cc;
-  int block_length;
-  char *insn;
-  int state_handled;
-};
-
 #ifdef OBJ_ELF
 # define obj_frob_symbol(sym, punt)	armelf_frob_symbol ((sym), & (punt))
 # define md_elf_section_change_hook()	arm_elf_change_section ()
@@ -272,13 +238,7 @@ void mapping_state (enum mstate);
 struct arm_segment_info_type
 {
   enum mstate mapstate;
-
-  /* Bit N indicates that an R_ARM_NONE relocation has been output for
-     __aeabi_unwind_cpp_prN already if set. This enables dependencies to be
-     emitted only once per section, to save unnecessary bloat.  */
   unsigned int marked_pr_dependency;
-
-  struct current_it current_it;
 };
 
 /* We want .cfi_* pseudo-ops for generating unwind info.  */
@@ -325,7 +285,7 @@ extern char * arm_canonicalize_symbol_name (char *);
 extern void arm_adjust_symtab (void);
 extern void armelf_frob_symbol (symbolS *, int *);
 extern void cons_fix_new_arm (fragS *, int, int, expressionS *);
-extern void arm_init_frag (struct frag *, int);
+extern void arm_init_frag (struct frag *);
 extern void arm_handle_align (struct frag *);
 extern bfd_boolean arm_fix_adjustable (struct fix *);
 extern int arm_elf_section_type (const char *, size_t);
