@@ -1,5 +1,5 @@
 /* tc-tic4x.c -- Assemble for the Texas Instruments TMS320C[34]x.
-   Copyright (C) 1997,1998, 2002, 2003, 2005, 2006, 2007, 2008, 2009
+   Copyright (C) 1997,1998, 2002, 2003, 2005, 2006, 2007, 2008
    Free Software Foundation. Inc.
 
    Contributed by Michael P. Hayes (m.hayes@elec.canterbury.ac.nz)
@@ -396,7 +396,7 @@ tic4x_gen_to_words (FLONUM_TYPE flonum, LITTLENUM_TYPE *words, int precision)
       || flonum.sign == 0) /* = NaN */
     {
       if(flonum.sign == 0)
-        as_bad (_("Nan, using zero."));
+        as_bad ("Nan, using zero.");
       words[0] = 0x8000;
       return return_value;
     }
@@ -518,7 +518,7 @@ tic4x_gen_to_words (FLONUM_TYPE flonum, LITTLENUM_TYPE *words, int precision)
     }
 
   if (abs (exponent) >= (1 << (exponent_bits - 1)))
-    as_bad (_("Cannot represent exponent in %d bits"), exponent_bits);
+    as_bad ("Cannot represent exponent in %d bits", exponent_bits);
 
   /* Force exponent to fit in desired field width.  */
   exponent &= (1 << (exponent_bits)) - 1;
@@ -601,7 +601,7 @@ tic4x_atof (char *str, char what_kind, LITTLENUM_TYPE *words)
       break;
 
     default:
-      as_bad (_("Invalid floating point number"));
+      as_bad ("Invalid floating point number");
       return (NULL);
     }
 
@@ -611,7 +611,7 @@ tic4x_atof (char *str, char what_kind, LITTLENUM_TYPE *words)
   if (atof_generic (&return_value, ".", EXP_CHARS,
 		    &generic_floating_point_number))
     {
-      as_bad (_("Invalid floating point number"));
+      as_bad ("Invalid floating point number");
       return (NULL);
     }
 
@@ -731,7 +731,7 @@ tic4x_asg (int x ATTRIBUTE_UNUSED)
     input_line_pointer++;
   if (*input_line_pointer != ',')
     {
-      as_bad (_("Comma expected\n"));
+      as_bad ("Comma expected\n");
       return;
     }
   *input_line_pointer++ = '\0';
@@ -771,7 +771,7 @@ tic4x_bss (int x ATTRIBUTE_UNUSED)
   c = get_symbol_end ();	/* Get terminator.  */
   if (c != ',')
     {
-      as_bad (_(".bss size argument missing\n"));
+      as_bad (".bss size argument missing\n");
       return;
     }
 
@@ -779,7 +779,7 @@ tic4x_bss (int x ATTRIBUTE_UNUSED)
     tic4x_expression_abs (++input_line_pointer, &size);
   if (size < 0)
     {
-      as_bad (_(".bss size %ld < 0!"), (long) size);
+      as_bad (".bss size %ld < 0!", (long) size);
       return;
     }
   subseg_set (bss_section, 0);
@@ -913,7 +913,7 @@ tic4x_stringer (int append_zero)
 	  input_line_pointer = tic4x_expression (input_line_pointer, &exp);
 	  if (exp.X_op != O_constant)
             {
-              as_bad (_("Non-constant symbols not allowed\n"));
+              as_bad("Non-constant symbols not allowed\n");
               return;
             }
           exp.X_add_number &= 255; /* Limit numeber to 8-bit */
@@ -944,7 +944,7 @@ tic4x_eval (int x ATTRIBUTE_UNUSED)
     tic4x_expression_abs (input_line_pointer, &value);
   if (*input_line_pointer++ != ',')
     {
-      as_bad (_("Symbol missing\n"));
+      as_bad ("Symbol missing\n");
       return;
     }
   name = input_line_pointer;
@@ -991,7 +991,7 @@ tic4x_sect (int x ATTRIBUTE_UNUSED)
       subsection_name = input_line_pointer;
       c = get_symbol_end ();	/* Get terminator.  */
       input_line_pointer++;	/* Skip null symbol terminator.  */
-      as_warn (_(".sect: subsection name ignored"));
+      as_warn (".sect: subsection name ignored");
     }
 
   /* We might still have a '"' to discard, but the character after a
@@ -1019,7 +1019,7 @@ tic4x_sect (int x ATTRIBUTE_UNUSED)
   if (bfd_get_section_flags (stdoutput, seg) == SEC_NO_FLAGS)
     {
       if (!bfd_set_section_flags (stdoutput, seg, SEC_DATA))
-	as_warn (_("Error setting flags for \"%s\": %s"), name,
+	as_warn ("Error setting flags for \"%s\": %s", name,
 		 bfd_errmsg (bfd_get_error ()));
     }
 
@@ -1050,7 +1050,7 @@ tic4x_set (int x ATTRIBUTE_UNUSED)
       c = get_symbol_end ();	/* Get terminator.  */
       if (c != ',')
 	{
-	  as_bad (_(".set syntax invalid\n"));
+	  as_bad (".set syntax invalid\n");
 	  ignore_rest_of_line ();
 	  return;
 	}
@@ -1108,7 +1108,7 @@ tic4x_usect (int x ATTRIBUTE_UNUSED)
   else
     alignment_flag = 0;
   if (alignment_flag)
-    as_warn (_(".usect: non-zero alignment flag ignored"));
+    as_warn (".usect: non-zero alignment flag ignored");
 
   seg = subseg_new (name, 0);
   if (line_label != NULL)
@@ -1119,7 +1119,7 @@ tic4x_usect (int x ATTRIBUTE_UNUSED)
     }
   seg_info (seg)->bss = 1;	/* Uninitialised data.  */
   if (!bfd_set_section_flags (stdoutput, seg, SEC_ALLOC))
-    as_warn (_("Error setting flags for \"%s\": %s"), name,
+    as_warn ("Error setting flags for \"%s\": %s", name,
 	     bfd_errmsg (bfd_get_error ()));
   tic4x_seg_alloc (name, seg, size, line_label);
 
@@ -1139,11 +1139,11 @@ tic4x_version (int x ATTRIBUTE_UNUSED)
   input_line_pointer =
     tic4x_expression_abs (input_line_pointer, &temp);
   if (!IS_CPU_TIC3X (temp) && !IS_CPU_TIC4X (temp))
-    as_bad (_("This assembler does not support processor generation %ld"),
+    as_bad ("This assembler does not support processor generation %ld",
 	    (long) temp);
 
   if (tic4x_cpu && temp != (offsetT) tic4x_cpu)
-    as_warn (_("Changing processor generation on fly not supported..."));
+    as_warn ("Changing processor generation on fly not supported...");
   tic4x_cpu = temp;
   demand_empty_rest_of_line ();
 }
@@ -1285,8 +1285,8 @@ tic4x_inst_add (const tic4x_inst_t *insts)
   d = name;
 
   /* We do not care about INSNs that is not a part of our
-     oplevel setting.  */
-  if ((insts->oplevel & tic4x_oplevel) == 0)
+     oplevel setting */
+  if (!insts->oplevel & tic4x_oplevel)
     return ok;
 
   while (1)
@@ -1442,7 +1442,7 @@ tic4x_indirect_parse (tic4x_operand_t *operand,
 	  if (operand->aregno >= REG_AR0 && operand->aregno <= REG_AR7)
 	    break;
 
-	  as_bad (_("Auxiliary register AR0--AR7 required for indirect"));
+	  as_bad ("Auxiliary register AR0--AR7 required for indirect");
 	  return -1;
 
 	case 'd':		/* Need to match constant for disp.  */
@@ -1456,7 +1456,7 @@ tic4x_indirect_parse (tic4x_operand_t *operand,
 	  operand->disp = operand->expr.X_add_number;
 	  if (operand->disp < 0 || operand->disp > 255)
 	    {
-	      as_bad (_("Bad displacement %d (require 0--255)\n"),
+	      as_bad ("Bad displacement %d (require 0--255)\n",
 		      operand->disp);
 	      return -1;
 	    }
@@ -1474,7 +1474,7 @@ tic4x_indirect_parse (tic4x_operand_t *operand,
 	  if (operand->expr.X_add_number != REG_IR0
 	      && operand->expr.X_add_number != REG_IR1)
 	    {
-	      as_bad (_("Index register IR0,IR1 required for displacement"));
+	      as_bad ("Index register IR0,IR1 required for displacement");
 	      return -1;
 	    }
 
@@ -1543,7 +1543,7 @@ tic4x_operand_parse (char *s, tic4x_operand_t *operand)
     case '%':
       input_line_pointer = tic4x_expression (++input_line_pointer, exp);
       if (exp->X_op != O_register)
-	as_bad (_("Expecting a register name"));
+	as_bad ("Expecting a register name");
       operand->mode = M_REGISTER;
       break;
 
@@ -1555,7 +1555,7 @@ tic4x_operand_parse (char *s, tic4x_operand_t *operand)
       else if (exp->X_op == O_big)
 	{
 	  if (exp->X_add_number)
-	    as_bad (_("Number too large"));	/* bignum required */
+	    as_bad ("Number too large");	/* bignum required */
 	  else
 	    {
 	      tic4x_gen_to_words (generic_floating_point_number,
@@ -1578,7 +1578,7 @@ tic4x_operand_parse (char *s, tic4x_operand_t *operand)
       else if (exp->X_op == O_big)
 	{
 	  if (exp->X_add_number > 0)
-	    as_bad (_("Number too large"));	/* bignum required.  */
+	    as_bad ("Number too large");	/* bignum required.  */
 	  else
 	    {
 	      tic4x_gen_to_words (generic_floating_point_number,
@@ -1595,18 +1595,18 @@ tic4x_operand_parse (char *s, tic4x_operand_t *operand)
 	}
 
       else
-	as_bad (_("Expecting a constant value"));
+	as_bad ("Expecting a constant value");
       break;
     case '\\':
 #endif
     case '@':
       input_line_pointer = tic4x_expression (++input_line_pointer, exp);
       if (exp->X_op != O_constant && exp->X_op != O_symbol)
-	as_bad (_("Bad direct addressing construct %s"), s);
+	as_bad ("Bad direct addressing construct %s", s);
       if (exp->X_op == O_constant)
 	{
 	  if (exp->X_add_number < 0)
-	    as_bad (_("Direct value of %ld is not suitable"),
+	    as_bad ("Direct value of %ld is not suitable",
 		    (long) exp->X_add_number);
 	}
       operand->mode = M_DIRECT;
@@ -1630,7 +1630,7 @@ tic4x_operand_parse (char *s, tic4x_operand_t *operand)
 	    operand->expr.X_add_number = 0x18;
 	}
       else
-	as_bad (_("Unknown indirect addressing mode"));
+	as_bad ("Unknown indirect addressing mode");
       break;
 
     default:
@@ -1647,7 +1647,7 @@ tic4x_operand_parse (char *s, tic4x_operand_t *operand)
       else if (exp->X_op == O_big)
 	{
 	  if (exp->X_add_number > 0)
-	    as_bad (_("Number too large"));	/* bignum required.  */
+	    as_bad ("Number too large");	/* bignum required.  */
 	  else
 	    {
 	      tic4x_gen_to_words (generic_floating_point_number,
@@ -1727,7 +1727,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
               else
                 {
 		  if (!check)
-                    as_bad (_("Immediate value of %ld is too large for ldf"),
+                    as_bad ("Immediate value of %ld is too large for ldf",
                             (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
@@ -1767,7 +1767,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Destination register must be ARn"));
+                as_bad ("Destination register must be ARn");
 	      ret = -1;
 	    }
 	  continue;
@@ -1786,7 +1786,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      else
 		{
 		  if (!check)
-                    as_bad (_("Immediate value of %ld is too large"),
+                    as_bad ("Immediate value of %ld is too large",
                             (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
@@ -1815,7 +1815,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      && operand->expr.X_add_number != 0x18)
 	    {
               if (!check)
-                as_bad (_("Invalid indirect addressing mode"));
+                as_bad ("Invalid indirect addressing mode");
               ret = -1;
 	      continue;
 	    }
@@ -1839,7 +1839,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be Rn"));
+                as_bad ("Register must be Rn");
 	      ret = -1;
 	    }
           continue;
@@ -1879,7 +1879,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be Rn"));
+                as_bad ("Register must be Rn");
 	      ret = -1;
 	    }
           continue;
@@ -1893,7 +1893,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be R0--R7"));
+                as_bad ("Register must be R0--R7");
 	      ret = -1;
 	    }
 	  continue;
@@ -1917,7 +1917,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      if (IS_CPU_TIC4X (tic4x_cpu))
 		break;
               if (!check)
-                as_bad (_("Invalid indirect addressing mode displacement %d"),
+                as_bad ("Invalid indirect addressing mode displacement %d",
                         operand->disp);
 	      ret = -1;
 	      continue;
@@ -1945,7 +1945,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      if (IS_CPU_TIC4X (tic4x_cpu))
 		break;
               if (!check)
-                as_bad (_("Invalid indirect addressing mode displacement %d"),
+                as_bad ("Invalid indirect addressing mode displacement %d",
                         operand->disp);
 	      ret = -1;
 	      continue;
@@ -1963,7 +1963,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be R0--R7"));
+                as_bad ("Register must be R0--R7");
 	      ret = -1;
 	    }
 	  continue;
@@ -1977,7 +1977,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be R0--R7"));
+                as_bad ("Register must be R0--R7");
 	      ret = -1;
 	    }
 	  continue;
@@ -1991,7 +1991,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Destination register must be R2 or R3"));
+                as_bad ("Destination register must be R2 or R3");
 	      ret = -1;
 	    }
 	  continue;
@@ -2005,7 +2005,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Destination register must be R0 or R1"));
+                as_bad ("Destination register must be R0 or R1");
 	      ret = -1;
 	    }
 	  continue;
@@ -2020,7 +2020,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      && operand->expr.X_add_number != 0x18)
 	    {
               if (!check)
-                as_bad (_("Invalid indirect addressing mode"));
+                as_bad ("Invalid indirect addressing mode");
 	      ret = -1;
 	      continue;
 	    }
@@ -2042,7 +2042,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      else
 		{
                   if (!check)
-                    as_bad (_("Displacement value of %ld is too large"),
+                    as_bad ("Displacement value of %ld is too large",
                             (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
@@ -2070,7 +2070,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be Rn"));
+                as_bad ("Register must be Rn");
 	      ret = -1;
 	    }
           continue;
@@ -2092,7 +2092,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be Rn"));
+                as_bad ("Register must be Rn");
 	      ret = -1;
 	    }
           continue;
@@ -2103,7 +2103,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  if (exp->X_op == O_big)
 	    {
               if (!check)
-                as_bad (_("Floating point number not valid in expression"));
+                as_bad ("Floating point number not valid in expression");
 	      ret = -1;
 	      continue;
 	    }
@@ -2117,7 +2117,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      else
 		{
 		  if (!check)
-                    as_bad (_("Signed immediate value %ld too large"),
+                    as_bad ("Signed immediate value %ld too large",
                             (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
@@ -2159,7 +2159,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      else
 		{
                   if (!check)
-                    as_bad (_("Immediate value of %ld is too large"),
+                    as_bad ("Immediate value of %ld is too large",
                             (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
@@ -2180,7 +2180,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      else
 		{
                   if (!check)
-                    as_bad (_("Unsigned immediate value %ld too large"),
+                    as_bad ("Unsigned immediate value %ld too large",
                             (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
@@ -2218,7 +2218,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      else
 		{
                   if (!check)
-                    as_bad (_("Immediate value of %ld is too large"),
+                    as_bad ("Immediate value of %ld is too large",
                             (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
@@ -2234,7 +2234,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  if (exp->X_op == O_big)
 	    {
               if (!check)
-                as_bad (_("Floating point number not valid in expression"));
+                as_bad ("Floating point number not valid in expression");
 	      ret = -1;
 	      continue;
 	    }
@@ -2248,7 +2248,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	      else
 		{
                   if (!check)
-                    as_bad (_("Immediate value %ld too large"),
+                    as_bad ("Immediate value %ld too large",
                             (long) exp->X_add_number);
 		  ret = -1;
 		  continue;
@@ -2267,7 +2267,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be ivtp or tvtp"));
+                as_bad ("Register must be ivtp or tvtp");
 	      ret = -1;
 	    }
 	  continue;
@@ -2281,7 +2281,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be address register"));
+                as_bad ("Register must be address register");
 	      ret = -1;
 	    }
 	  continue;
@@ -2295,7 +2295,7 @@ tic4x_operands_match (tic4x_inst_t *inst, tic4x_insn_t *insn, int check)
 	  else
 	    {
               if (!check)
-                as_bad (_("Register must be ivtp or tvtp"));
+                as_bad ("Register must be ivtp or tvtp");
 	      ret = -1;
 	    }
 	  continue;
@@ -2354,7 +2354,7 @@ tic4x_insn_check (tic4x_insn_t *insn)
       if ( insn->operands[0].mode == M_REGISTER
            && insn->operands[1].mode == M_REGISTER
            && insn->operands[0].expr.X_add_number == insn->operands[1].expr.X_add_number )
-        as_bad (_("Source and destination register should not be equal"));
+        as_bad ("Source and destination register should not be equal");
     }
   else if( !strcmp(insn->name, "ldi_ldi")
            || !strcmp(insn->name, "ldi1_ldi2")
@@ -2369,7 +2369,7 @@ tic4x_insn_check (tic4x_insn_t *insn)
       if ( insn->operands[1].mode == M_REGISTER
            && insn->operands[insn->num_operands-1].mode == M_REGISTER
            && insn->operands[1].expr.X_add_number == insn->operands[insn->num_operands-1].expr.X_add_number )
-        as_warn (_("Equal parallell destination registers, one result will be discarded"));
+        as_warn ("Equal parallell destination registers, one result will be discarded");
     }
 }
 
@@ -2410,7 +2410,7 @@ tic4x_operands_parse (char *s, tic4x_operand_t *operands, int num_operands)
 
   if (num_operands > TIC4X_OPERANDS_MAX)
     {
-      as_bad (_("Too many operands scanned"));
+      as_bad ("Too many operands scanned");
       return -1;
     }
   return num_operands;
@@ -2440,7 +2440,7 @@ md_assemble (char *str)
         {
           if(insn->parallel)
             {
-              as_bad (_("Parallel opcode cannot contain more than two instructions"));
+              as_bad ("Parallel opcode cannot contain more than two instructions");
               insn->parallel = 0;
               insn->in_use = 0;
               return;
@@ -2485,7 +2485,7 @@ md_assemble (char *str)
       if ((insn->inst = (struct tic4x_inst *)
 	   hash_find (tic4x_op_hash, insn->name)) == NULL)
 	{
-	  as_bad (_("Unknown opcode `%s'."), insn->name);
+	  as_bad ("Unknown opcode `%s'.", insn->name);
 	  insn->parallel = 0;
 	  insn->in_use = 0;
 	  return;
@@ -2513,10 +2513,10 @@ md_assemble (char *str)
         {
           if (first_inst)
             tic4x_operands_match (first_inst, insn, 0);
-          as_bad (_("Invalid operands for %s"), insn->name);
+          as_bad ("Invalid operands for %s", insn->name);
         }
       else
-	as_bad (_("Invalid instruction %s"), insn->name);
+	as_bad ("Invalid instruction %s", insn->name);
     }
 
   if (str && !parsed)
@@ -2665,7 +2665,7 @@ md_apply_fix (fixS *fixP, valueT *value, segT seg ATTRIBUTE_UNUSED)
 
     case NO_RELOC:
     default:
-      as_bad (_("Bad relocation type: 0x%02x"), fixP->fx_r_type);
+      as_bad ("Bad relocation type: 0x%02x", fixP->fx_r_type);
       break;
     }
 
@@ -2723,7 +2723,7 @@ md_parse_option (int c, char *arg)
 	arg++;
       tic4x_cpu = atoi (arg);
       if (!IS_CPU_TIC3X (tic4x_cpu) && !IS_CPU_TIC4X (tic4x_cpu))
-	as_warn (_("Unsupported processor generation %d"), tic4x_cpu);
+	as_warn ("Unsupported processor generation %d", tic4x_cpu);
       break;
 
     case OPTION_REV:             /* cpu revision */
@@ -2731,25 +2731,25 @@ md_parse_option (int c, char *arg)
       break;
 
     case 'b':
-      as_warn (_("Option -b is depreciated, please use -mbig"));
+      as_warn ("Option -b is depreciated, please use -mbig");
     case OPTION_BIG:             /* big model */
       tic4x_big_model = 1;
       break;
 
     case 'p':
-      as_warn (_("Option -p is depreciated, please use -mmemparm"));
+      as_warn ("Option -p is depreciated, please use -mmemparm");
     case OPTION_MEMPARM:         /* push args */
       tic4x_reg_args = 0;
       break;
 
     case 'r':			
-      as_warn (_("Option -r is depreciated, please use -mregparm"));
+      as_warn ("Option -r is depreciated, please use -mregparm");
     case OPTION_REGPARM:        /* register args */
       tic4x_reg_args = 1;
       break;
 
     case 's':
-      as_warn (_("Option -s is depreciated, please use -msmall"));
+      as_warn ("Option -s is depreciated, please use -msmall");
     case OPTION_SMALL:		/* small model */
       tic4x_big_model = 0;
       break;
@@ -2822,7 +2822,7 @@ tic4x_unrecognized_line (int c)
 
   if (dollar_label_defined (lab))
     {
-      as_bad (_("Label \"$%d\" redefined"), lab);
+      as_bad ("Label \"$%d\" redefined", lab);
       return 0;
     }
 
@@ -3030,7 +3030,7 @@ tc_gen_reloc (asection *seg ATTRIBUTE_UNUSED, fixS *fixP)
   if (reloc->howto == (reloc_howto_type *) NULL)
     {
       as_bad_where (fixP->fx_file, fixP->fx_line,
-		    _("Reloc %d not supported by object file format"),
+		    "Reloc %d not supported by object file format",
 		    (int) fixP->fx_r_type);
       return NULL;
     }
