@@ -74,14 +74,18 @@ class Output_data_plt_arm;
 // R_ARM_TARGET1
 // R_ARM_PREL31
 // 
-// TODOs:
-// - Generate various branch stubs.
+// Coming soon (pending patches):
+// - Defining section symbols __exidx_start and __exidx_stop.
 // - Support interworking.
-// - Define section symbols __exidx_start and __exidx_stop.
+// - Mergeing all .ARM.xxx.yyy sections into .ARM.xxx.  Currently, they
+//   are incorrectly merged into an .ARM section.
+//
+// TODOs:
+// - Create a PT_ARM_EXIDX program header for a shared object that
+//   might throw an exception.
 // - Support more relocation types as needed. 
 // - Make PLTs more flexible for different architecture features like
 //   Thumb-2 and BE8.
-// There are probably a lot more.
 
 // Utilities for manipulating integers of up to 32-bits
 
@@ -445,11 +449,7 @@ const Target::Target_info Target_arm<big_endian>::arm_info =
   "/usr/lib/libc.so.1",	// dynamic_linker
   0x8000,		// default_text_segment_address
   0x1000,		// abi_pagesize (overridable by -z max-page-size)
-  0x1000,		// common_pagesize (overridable by -z common-page-size)
-  elfcpp::SHN_UNDEF,	// small_common_shndx
-  elfcpp::SHN_UNDEF,	// large_common_shndx
-  0,			// small_common_section_flags
-  0			// large_common_section_flags
+  0x1000		// common_pagesize (overridable by -z common-page-size)
 };
 
 // Arm relocate functions class
@@ -1459,25 +1459,6 @@ Target_arm<big_endian>::do_finalize_sections(Layout* layout)
   // relocs.
   if (this->copy_relocs_.any_saved_relocs())
     this->copy_relocs_.emit(this->rel_dyn_section(layout));
-
-  // For the ARM target, we need to add a PT_ARM_EXIDX segment for
-  // the .ARM.exidx section.
-  if (!layout->script_options()->saw_phdrs_clause()
-      && !parameters->options().relocatable())
-    {
-      Output_section* exidx_section =
-	layout->find_output_section(".ARM.exidx");
-
-      if (exidx_section != NULL
-	  && exidx_section->type() == elfcpp::SHT_ARM_EXIDX)
-	{
-	  gold_assert(layout->find_output_segment(elfcpp::PT_ARM_EXIDX, 0, 0)
-		      == NULL);
-	  Output_segment*  exidx_segment =
-	    layout->make_output_segment(elfcpp::PT_ARM_EXIDX, elfcpp::PF_R);
-	  exidx_segment->add_output_section(exidx_section, elfcpp::PF_R);
-	}
-    }
 }
 
 // Return whether a direct absolute static relocation needs to be applied.
