@@ -161,9 +161,9 @@ class Sized_dynobj : public Dynobj
   Sized_dynobj(const std::string& name, Input_file* input_file, off_t offset,
 	       const typename elfcpp::Ehdr<size, big_endian>&);
 
-  // Set up the object file based on TARGET.
+  // Set up the object file based on the ELF header.
   void
-  setup(Target *target);
+  setup(const typename elfcpp::Ehdr<size, big_endian>&);
 
   // Read the symbols.
   void
@@ -198,11 +198,6 @@ class Sized_dynobj : public Dynobj
   do_section_flags(unsigned int shndx)
   { return this->elf_file_.section_flags(shndx); }
 
-  // Not used for dynobj.
-  uint64_t
-  do_section_entsize(unsigned int )
-  { gold_unreachable(); }
-
   // Return section address.
   uint64_t
   do_section_address(unsigned int shndx)
