@@ -1,6 +1,6 @@
 // fileread.h -- read files for gold   -*- C++ -*-
 
-// Copyright 2006, 2007, 2008 Free Software Foundation, Inc.
+// Copyright 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // Written by Ian Lance Taylor <iant@google.com>.
 
 // This file is part of gold.
@@ -109,11 +109,6 @@ class File_read
   void
   release();
 
-  // Claim the file for a plugin.  This effectively releases the file without
-  // closing it; the plugin will assume responsibility for closing it.
-  void
-  claim_for_plugin();
-
   // Return the size of the file.
   off_t
   filesize() const
@@ -190,9 +185,9 @@ class File_read
 
   // Return the open file descriptor (for plugins).
   int
-  descriptor() const
+  descriptor()
   {
-    gold_assert(this->descriptor_ >= 0);
+    this->reopen_descriptor();
     return this->descriptor_;
   }
 
@@ -436,10 +431,23 @@ class Input_file
   Input_file(const Task*, const char* name, const unsigned char* contents,
 	     off_t size);
 
-  // Open the file.  If the open fails, this will report an error and
-  // return false.
+  // Return the command line argument.
+  const Input_file_argument*
+  input_file_argument() const
+  { return this->input_argument_; }
+
+  // Return whether this is a file that we will search for in the list
+  // of directories.
   bool
-  open(const General_options&, const Dirsearch&, const Task*);
+  will_search_for() const;
+
+  // Open the file.  If the open fails, this will report an error and
+  // return false.  If there is a search, it starts at directory
+  // *PINDEX.  *PINDEX should be initialized to zero.  It may be
+  // restarted to find the next file with a matching name by
+  // incrementing the result and calling this again.
+  bool
+  open(const Dirsearch&, const Task*, int *pindex);
 
   // Return the name given by the user.  For -lc this will return "c".
   const char*
@@ -476,6 +484,10 @@ class Input_file
   is_in_sysroot() const
   { return this->is_in_sysroot_; }
 
+  // Whether this file is in a system directory.
+  bool
+  is_in_system_directory() const;
+
   // Return whether this file is to be read only for its symbols.
   bool
   just_symbols() const;
@@ -486,8 +498,7 @@ class Input_file
 
   // Open a binary file.
   bool
-  open_binary(const General_options&, const Task* task,
-	      const std::string& name);
+  open_binary(const Task* task, const std::string& name);
 
   // The argument from the command line.
   const Input_file_argument* input_argument_;
