@@ -41,7 +41,6 @@
 #include "reloc.h"
 #include "defstd.h"
 #include "plugin.h"
-#include "icf.h"
 
 namespace gold
 {
@@ -165,11 +164,7 @@ queue_initial_tasks(const General_options& options,
 		    Symbol_table* symtab, Layout* layout, Mapfile* mapfile)
 {
   if (cmdline.begin() == cmdline.end())
-    {
-      if (options.printed_version())
-	gold_exit(true);
-      gold_fatal(_("no input files"));
-    }
+    gold_fatal(_("no input files"));
 
   int thread_count = options.thread_count_initial();
   if (thread_count == 0)
@@ -204,10 +199,10 @@ queue_initial_tasks(const General_options& options,
     }
 
   if (parameters->options().relocatable()
-      && (parameters->options().gc_sections() || parameters->options().icf()))
-    gold_error(_("cannot mix -r with --gc-sections or --icf"));
+      && parameters->options().gc_sections())
+    gold_error(_("cannot mix -r with garbage collection"));
 
-  if (parameters->options().gc_sections() || parameters->options().icf())
+  if (parameters->options().gc_sections())
     {
       workqueue->queue(new Task_function(new Gc_runner(options,
                                                        input_objects,
@@ -310,23 +305,8 @@ queue_middle_tasks(const General_options& options,
       gold_assert(symtab->gc() != NULL);
       // Do a transitive closure on all references to determine the worklist.
       symtab->gc()->do_transitive_closure();
-    }
-
-  // If identical code folding (--icf) is chosen it makes sense to do it 
-  // only after garbage collection (--gc-sections) as we do not want to 
-  // be folding sections that will be garbage.
-  if (parameters->options().icf())
-    {
-      symtab->icf()->find_identical_sections(input_objects, symtab);
-    }
-
-  // Call Object::layout for the second time to determine the 
-  // output_sections for all referenced input sections.  When 
-  // --gc-sections or --icf is turned on, Object::layout is 
-  // called twice.  It is called the first time when the 
-  // symbols are added.
-  if (parameters->options().gc_sections() || parameters->options().icf())
-    {
+      // Call do_layout again to determine the output_sections for all 
+      // referenced input sections.
       for (Input_objects::Relobj_iterator p = input_objects->relobj_begin();
            p != input_objects->relobj_end();
            ++p)
@@ -334,7 +314,6 @@ queue_middle_tasks(const General_options& options,
           (*p)->layout(symtab, layout, NULL);
         }
     }
-
   // Layout deferred objects due to plugins.
   if (parameters->options().has_plugins())
     {
@@ -342,8 +321,7 @@ queue_middle_tasks(const General_options& options,
       gold_assert(plugins != NULL);
       plugins->layout_deferred_objects();
     }     
-
-  if (parameters->options().gc_sections() || parameters->options().icf())
+  if (parameters->options().gc_sections())
     {
       for (Input_objects::Relobj_iterator p = input_objects->relobj_begin();
            p != input_objects->relobj_end();
@@ -386,7 +364,7 @@ queue_middle_tasks(const General_options& options,
 		 (*input_objects->dynobj_begin())->name().c_str());
     }
   if (!doing_static_link && parameters->options().relocatable())
-    gold_fatal(_("cannot mix -r with dynamic object %s"),
+    gold_error(_("cannot mix -r with dynamic object %s"),
 	       (*input_objects->dynobj_begin())->name().c_str());
   if (!doing_static_link
       && options.oformat_enum() != General_options::OBJECT_FORMAT_ELF)
@@ -404,9 +382,6 @@ queue_middle_tasks(const General_options& options,
   // TODO: if this is too slow, do this as a task, rather than inline.
   symtab->detect_odr_violations(task, options.output_file_name());
 
-  // Create any automatic note sections.
-  layout->create_notes();
-
   // Create any output sections required by any linker script.
   layout->create_script_sections();
 
@@ -438,7 +413,7 @@ queue_middle_tasks(const General_options& options,
 
   // If doing garbage collection, the relocations have already been read.
   // Otherwise, read and scan the relocations.
-  if (parameters->options().gc_sections() || parameters->options().icf())
+  if (parameters->options().gc_sections())
     {
       for (Input_objects::Relobj_iterator p = input_objects->relobj_begin();
            p != input_objects->relobj_end();
