@@ -132,6 +132,19 @@ Xindex::sym_xindex_to_shndx(Object* object, unsigned int symndx)
 
 // Class Object.
 
+// Set the target based on fields in the ELF file header.
+
+void
+Object::set_target(int machine, int size, bool big_endian, int osabi,
+		   int abiversion)
+{
+  Target* target = select_target(machine, size, big_endian, osabi, abiversion);
+  if (target == NULL)
+    gold_fatal(_("%s: unsupported ELF machine number %d"),
+	       this->name().c_str(), machine);
+  this->target_ = target;
+}
+
 // Report an error for this object file.  This is used by the
 // elfcpp::Elf_file interface, and also called by the Object code
 // itself.
@@ -157,11 +170,6 @@ Object::section_contents(unsigned int shndx, section_size_type* plen,
 {
   Location loc(this->do_section_contents(shndx));
   *plen = convert_to_section_size_type(loc.data_size);
-  if (*plen == 0)
-    {
-      static const unsigned char empty[1] = { '\0' };
-      return empty;
-    }
   return this->get_view(loc.file_offset, *plen, true, cache);
 }
 
@@ -218,12 +226,6 @@ Object::handle_gnu_warning_section(const char* name, unsigned int shndx,
       section_size_type len;
       const unsigned char* contents = this->section_contents(shndx, &len,
 							     false);
-      if (len == 0)
-	{
-	  const char* warning = name + warn_prefix_len;
-	  contents = reinterpret_cast<const unsigned char*>(warning);
-	  len = strlen(warning);
-	}
       std::string warning(reinterpret_cast<const char*>(contents), len);
       symtab->add_warning(name + warn_prefix_len, this, warning);
       return true;
@@ -340,9 +342,12 @@ Sized_relobj<size, big_endian>::~Sized_relobj()
 
 template<int size, bool big_endian>
 void
-Sized_relobj<size, big_endian>::setup(Target *target)
+Sized_relobj<size, big_endian>::setup(
+    const elfcpp::Ehdr<size, big_endian>& ehdr)
 {
-  this->set_target(target);
+  this->set_target(ehdr.get_e_machine(), size, big_endian,
+		   ehdr.get_e_ident()[elfcpp::EI_OSABI],
+		   ehdr.get_e_ident()[elfcpp::EI_ABIVERSION]);
 
   const unsigned int shnum = this->elf_file_.shnum();
   this->set_shnum(shnum);
@@ -670,20 +675,24 @@ Sized_relobj<size, big_endian>::include_section_group(
   // Record this section group in the layout, and see whether we've already
   // seen one with the same signature.
   bool include_group;
-  bool is_comdat;
-  Kept_section* kept_section = NULL;
+  Sized_relobj<size, big_endian>* kept_object = NULL;
+  Kept_section::Comdat_group* kept_group = NULL;
 
   if ((flags & elfcpp::GRP_COMDAT) == 0)
-    {
-      include_group = true;
-      is_comdat = false;
-    }
+    include_group = true;
   else
     {
+      Kept_section this_group(this, index, true);
+      Kept_section *kept_section_group;
       include_group = layout->find_or_add_kept_section(signature,
-						       this, index, true,
-						       true, &kept_section);
-      is_comdat = true;
+                                                       &this_group,
+                                                       &kept_section_group);
+      if (include_group)
+        kept_section_group->group_sections = new Kept_section::Comdat_group;
+
+      kept_group = kept_section_group->group_sections;
+      kept_object = (static_cast<Sized_relobj<size, big_endian>*>
+		     (kept_section_group->object));
     }
 
   size_t count = shdr.get_sh_size() / sizeof(elfcpp::Elf_Word);
@@ -695,27 +704,27 @@ Sized_relobj<size, big_endian>::include_section_group(
 
   for (size_t i = 1; i < count; ++i)
     {
-      elfcpp::Elf_Word shndx =
+      elfcpp::Elf_Word secnum =
 	this->adjust_shndx(elfcpp::Swap<32, big_endian>::readval(pword + i));
 
       if (relocate_group)
-	shndxes.push_back(shndx);
+	shndxes.push_back(secnum);
 
-      if (shndx >= this->shnum())
+      if (secnum >= this->shnum())
 	{
 	  this->error(_("section %u in section group %u out of range"),
-		      shndx, index);
+		      secnum, index);
 	  continue;
 	}
 
       // Check for an earlier section number, since we're going to get
       // it wrong--we may have already decided to include the section.
-      if (shndx < index)
+      if (secnum < index)
         this->error(_("invalid section group %u refers to earlier section %u"),
-                    index, shndx);
+                    index, secnum);
 
       // Get the name of the member section.
-      typename This::Shdr member_shdr(shdrs + shndx * This::shdr_size);
+      typename This::Shdr member_shdr(shdrs + secnum * This::shdr_size);
       if (member_shdr.get_sh_name() >= section_names_size)
         {
           // This is an error, but it will be diagnosed eventually
@@ -725,53 +734,29 @@ Sized_relobj<size, big_endian>::include_section_group(
         }
       std::string mname(section_names + member_shdr.get_sh_name());
 
-      if (include_group)
-	{
-	  if (is_comdat)
-	    kept_section->add_comdat_section(mname, shndx,
-					     member_shdr.get_sh_size());
-	}
-      else
+      if (!include_group)
         {
-          (*omit)[shndx] = true;
-
-	  if (is_comdat)
+          (*omit)[secnum] = true;
+          if (kept_group != NULL)
             {
-	      Relobj* kept_object = kept_section->object();
-	      if (kept_section->is_comdat())
-		{
-		  // Find the corresponding kept section, and store
-		  // that info in the discarded section table.
-		  unsigned int kept_shndx;
-		  uint64_t kept_size;
-		  if (kept_section->find_comdat_section(mname, &kept_shndx,
-							&kept_size))
-		    {
-		      // We don't keep a mapping for this section if
-		      // it has a different size.  The mapping is only
-		      // used for relocation processing, and we don't
-		      // want to treat the sections as similar if the
-		      // sizes are different.  Checking the section
-		      // size is the approach used by the GNU linker.
-		      if (kept_size == member_shdr.get_sh_size())
-			this->set_kept_comdat_section(shndx, kept_object,
-						      kept_shndx);
-		    }
-		}
-	      else
-		{
-		  // The existing section is a linkonce section.  Add
-		  // a mapping if there is exactly one section in the
-		  // group (which is true when COUNT == 2) and if it
-		  // is the same size.
-		  if (count == 2
-		      && (kept_section->linkonce_size()
-			  == member_shdr.get_sh_size()))
-		    this->set_kept_comdat_section(shndx, kept_object,
-						  kept_section->shndx());
-		}
+              // Find the corresponding kept section, and store that info
+              // in the discarded section table.
+              Kept_section::Comdat_group::const_iterator p =
+                kept_group->find(mname);
+              if (p != kept_group->end())
+                {
+                  Kept_comdat_section* kept =
+                    new Kept_comdat_section(kept_object, p->second);
+                  this->set_kept_comdat_section(secnum, kept);
+                }
             }
         }
+      else if (flags & elfcpp::GRP_COMDAT)
+        {
+          // Add the section to the kept group table.
+          gold_assert(kept_group != NULL);
+          kept_group->insert(std::make_pair(mname, secnum));
+        }
     }
 
   if (relocate_group)
@@ -802,9 +787,8 @@ Sized_relobj<size, big_endian>::include_linkonce_section(
     Layout* layout,
     unsigned int index,
     const char* name,
-    const elfcpp::Shdr<size, big_endian>& shdr)
+    const elfcpp::Shdr<size, big_endian>&)
 {
-  typename elfcpp::Elf_types<size>::Elf_WXword sh_size = shdr.get_sh_size();
   // In general the symbol name we want will be the string following
   // the last '.'.  However, we have to handle the case of
   // .gnu.linkonce.t.__i686.get_pc_thunk.bx, which was generated by
@@ -821,24 +805,29 @@ Sized_relobj<size, big_endian>::include_linkonce_section(
     symname = strrchr(name, '.') + 1;
   std::string sig1(symname);
   std::string sig2(name);
+  Kept_section candidate1(this, index, false);
+  Kept_section candidate2(this, index, true);
   Kept_section* kept1;
   Kept_section* kept2;
-  bool include1 = layout->find_or_add_kept_section(sig1, this, index, false,
-						   false, &kept1);
-  bool include2 = layout->find_or_add_kept_section(sig2, this, index, false,
-						   true, &kept2);
+  bool include1 = layout->find_or_add_kept_section(sig1, &candidate1, &kept1);
+  bool include2 = layout->find_or_add_kept_section(sig2, &candidate2, &kept2);
 
   if (!include2)
     {
-      // We are not including this section because we already saw the
-      // name of the section as a signature.  This normally implies
-      // that the kept section is another linkonce section.  If it is
-      // the same size, record it as the section which corresponds to
-      // this one.
-      if (kept2->object() != NULL
-	  && !kept2->is_comdat()
-	  && kept2->linkonce_size() == sh_size)
-	this->set_kept_comdat_section(index, kept2->object(), kept2->shndx());
+      // The section is being discarded on the basis of its section
+      // name (i.e., the kept section was also a linkonce section).
+      // In this case, the section index stored with the layout object
+      // is the linkonce section that was kept.
+      unsigned int kept_group_index = kept2->shndx;
+      Relobj* kept_relobj = kept2->object;
+      if (kept_relobj != NULL)
+        {
+          Sized_relobj<size, big_endian>* kept_object =
+	    static_cast<Sized_relobj<size, big_endian>*>(kept_relobj);
+          Kept_comdat_section* kept =
+            new Kept_comdat_section(kept_object, kept_group_index);
+          this->set_kept_comdat_section(index, kept);
+        }
     }
   else if (!include1)
     {
@@ -849,18 +838,22 @@ Sized_relobj<size, big_endian>::include_linkonce_section(
       // this linkonce section.  We'll handle the simple case where
       // the group has only one member section.  Otherwise, it's not
       // worth the effort.
-      unsigned int kept_shndx;
-      uint64_t kept_size;
-      if (kept1->object() != NULL
-	  && kept1->is_comdat()
-	  && kept1->find_single_comdat_section(&kept_shndx, &kept_size)
-	  && kept_size == sh_size)
-	this->set_kept_comdat_section(index, kept1->object(), kept_shndx);
-    }
-  else
-    {
-      kept1->set_linkonce_size(sh_size);
-      kept2->set_linkonce_size(sh_size);
+      Relobj* kept_relobj = kept1->object;
+      if (kept_relobj != NULL)
+        {
+          Sized_relobj<size, big_endian>* kept_object =
+	    static_cast<Sized_relobj<size, big_endian>*>(kept_relobj);
+          Kept_section::Comdat_group* kept_group = kept1->group_sections;
+          if (kept_group != NULL && kept_group->size() == 1)
+            {
+              Kept_section::Comdat_group::const_iterator p =
+		kept_group->begin();
+              gold_assert(p != kept_group->end());
+              Kept_comdat_section* kept =
+                new Kept_comdat_section(kept_object, p->second);
+              this->set_kept_comdat_section(index, kept);
+            }
+        }
     }
 
   return include1 && include2;
@@ -898,15 +891,14 @@ Sized_relobj<size, big_endian>::layout_section(Layout* layout,
 // whether they should be included in the link.  If they should, we
 // pass them to the Layout object, which will return an output section
 // and an offset.  
-// During garbage collection (--gc-sections) and identical code folding 
-// (--icf), this function is called twice.  When it is called the first 
-// time, it is for setting up some sections as roots to a work-list for
-// --gc-sections and to do comdat processing.  Actual layout happens the 
-// second time around after all the relevant sections have been determined.  
-// The first time, is_worklist_ready or is_icf_ready is false. It is then 
-// set to true after the garbage collection worklist or identical code 
-// folding is processed and the relevant sections to be kept are 
-// determined.  Then, this function is called again to layout the sections.
+// During garbage collection (gc-sections), this function is called
+// twice.  When it is called the first time, it is for setting up some
+// sections as roots to a work-list and to do comdat processing.  Actual
+// layout happens the second time around after all the relevant sections
+// have been determined.  The first time, is_worklist_ready is false.  
+// It is then set to true after the worklist is processed and the relevant 
+// sections are determined.  Then, this function is called again to 
+// layout the sections.
 
 template<int size, bool big_endian>
 void
@@ -915,22 +907,10 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
 					  Read_symbols_data* sd)
 {
   const unsigned int shnum = this->shnum();
-  bool is_gc_pass_one = ((parameters->options().gc_sections() 
-                          && !symtab->gc()->is_worklist_ready())
-                         || (parameters->options().icf()
-                             && !symtab->icf()->is_icf_ready()));
- 
-  bool is_gc_pass_two = ((parameters->options().gc_sections() 
-                          && symtab->gc()->is_worklist_ready())
-                         || (parameters->options().icf()
-                             && symtab->icf()->is_icf_ready()));
-
-  bool is_gc_or_icf = (parameters->options().gc_sections()
-                       || parameters->options().icf()); 
-
-  // Both is_gc_pass_one and is_gc_pass_two should not be true.
-  gold_assert(!(is_gc_pass_one  && is_gc_pass_two));
-
+  bool is_gc_pass_one = (parameters->options().gc_sections() 
+                         && !symtab->gc()->is_worklist_ready());
+  bool is_gc_pass_two = (parameters->options().gc_sections() 
+                         && symtab->gc()->is_worklist_ready());
   if (shnum == 0)
     return;
   Symbols_data* gc_sd = NULL;
@@ -955,7 +935,7 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
   const unsigned char* symbol_names_data = NULL;
   section_size_type symbol_names_size;
  
-  if (is_gc_or_icf)
+  if (parameters->options().gc_sections())
     {
       section_headers_data = gc_sd->section_headers_data;
       section_names_size = gc_sd->section_names_size;
@@ -983,10 +963,9 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
   const unsigned char* pshdrs;
 
   // Get the section names.
-  const unsigned char* pnamesu = (is_gc_or_icf) 
-                                 ? gc_sd->section_names_data
-                                 : sd->section_names->data();
-
+  const unsigned char* pnamesu = parameters->options().gc_sections() ?
+                                 gc_sd->section_names_data :
+                                 sd->section_names->data();
   const char* pnames = reinterpret_cast<const char*>(pnamesu);
 
   // If any input files have been claimed by plugins, we need to defer
@@ -1139,7 +1118,7 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
             }
         }
  
-      if (is_gc_pass_one && parameters->options().gc_sections())
+      if (is_gc_pass_one)
         {
           if (is_section_name_included(name)
               || shdr.get_sh_type() == elfcpp::SHT_INIT_ARRAY 
@@ -1186,7 +1165,7 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
           continue;
         }
 
-      if (is_gc_pass_two && parameters->options().gc_sections())
+      if (is_gc_pass_two)
         {
           // This is executed during the second pass of garbage 
           // collection. do_layout has been called before and some 
@@ -1197,12 +1176,13 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
               gold_assert(out_section_offsets[i] == invalid_address);
               continue; 
             }
-          if (((shdr.get_sh_flags() & elfcpp::SHF_ALLOC) != 0)
-              && symtab->gc()->is_section_garbage(this, i))
+          if ((shdr.get_sh_flags() & elfcpp::SHF_ALLOC) != 0)
+            if (symtab->gc()->referenced_list().find(Section_id(this,i)) 
+                == symtab->gc()->referenced_list().end())
               {
                 if (parameters->options().print_gc_sections())
                   gold_info(_("%s: removing unused section from '%s'" 
-                              " in file '%s'"),
+                              " in file '%s"),
                             program_name, this->section_name(i).c_str(), 
                             this->name().c_str());
                 out_sections[i] = NULL;
@@ -1210,36 +1190,6 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
                 continue;
               }
         }
-
-      if (is_gc_pass_two && parameters->options().icf())
-        {
-          if (out_sections[i] == NULL)
-            {
-              gold_assert(out_section_offsets[i] == invalid_address);
-              continue;
-            }
-          if (((shdr.get_sh_flags() & elfcpp::SHF_ALLOC) != 0)
-              && symtab->icf()->is_section_folded(this, i))
-              {
-                if (parameters->options().print_icf_sections())
-                  {
-                    Section_id folded =
-                                symtab->icf()->get_folded_section(this, i);
-                    Relobj* folded_obj =
-                                reinterpret_cast<Relobj*>(folded.first);
-                    gold_info(_("%s: ICF folding section '%s' in file '%s'"
-                                "into '%s' in file '%s'"),
-                              program_name, this->section_name(i).c_str(),
-                              this->name().c_str(),
-                              folded_obj->section_name(folded.second).c_str(),
-                              folded_obj->name().c_str());
-                  }
-                out_sections[i] = NULL;
-                out_section_offsets[i] = invalid_address;
-                continue;
-              }
-        }
-
       // Defer layout here if input files are claimed by plugins.  When gc
       // is turned on this function is called twice.  For the second call
       // should_defer_layout should be false.
@@ -1255,8 +1205,7 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
           out_sections[i] = reinterpret_cast<Output_section*>(2);
           out_section_offsets[i] = invalid_address;
           continue;
-        }
-
+              }
       // During gc_pass_two if a section that was previously deferred is
       // found, do not layout the section as layout_deferred_sections will
       // do it later from gold.cc.
@@ -1284,13 +1233,10 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
     layout->layout_gnu_stack(seen_gnu_stack, gnu_stack_flags);
 
   // When doing a relocatable link handle the reloc sections at the
-  // end.  Garbage collection  and Identical Code Folding is not 
-  // turned on for relocatable code. 
+  // end.  Garbage collection is not turned on for relocatable code. 
   if (emit_relocs)
     this->size_relocatable_relocs();
-
-  gold_assert(!(is_gc_or_icf) || reloc_sections.empty());
-
+  gold_assert(!parameters->options().gc_sections() || reloc_sections.empty());
   for (std::vector<unsigned int>::const_iterator p = reloc_sections.begin();
        p != reloc_sections.end();
        ++p)
@@ -1373,7 +1319,6 @@ Sized_relobj<size, big_endian>::do_layout(Symbol_table* symtab,
       delete[] gc_sd->section_names_data;
       delete[] gc_sd->symbols_data;
       delete[] gc_sd->symbol_names_data;
-      this->set_symbols_data(NULL);
     }
   else
     {
@@ -1586,8 +1531,7 @@ Sized_relobj<size, big_endian>::do_count_local_symbols(Stringpool* pool,
 template<int size, bool big_endian>
 unsigned int
 Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
-							  off_t off,
-                                                          Symbol_table* symtab)
+							  off_t off)
 {
   gold_assert(off == static_cast<off_t>(align_address(off, size >> 3)));
 
@@ -1610,7 +1554,7 @@ Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
 
       if (!is_ordinary)
 	{
-	  if (shndx == elfcpp::SHN_ABS || Symbol::is_common_shndx(shndx))
+	  if (shndx == elfcpp::SHN_ABS || shndx == elfcpp::SHN_COMMON)
 	    lv.set_output_value(lv.input_value());
 	  else
 	    {
@@ -1629,21 +1573,6 @@ Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
 	    }
 
 	  Output_section* os = out_sections[shndx];
-          Address secoffset = out_offsets[shndx];
-          if (symtab->is_section_folded(this, shndx))
-            {
-              gold_assert (os == NULL && secoffset == invalid_address);
-              // Get the os of the section it is folded onto.
-              Section_id folded = symtab->icf()->get_folded_section(this,
-                                                                    shndx);
-              gold_assert(folded.first != NULL);
-              Sized_relobj<size, big_endian>* folded_obj = reinterpret_cast
-                <Sized_relobj<size, big_endian>*>(folded.first);
-              os = folded_obj->output_section(folded.second);
-              gold_assert(os != NULL);
-              secoffset = folded_obj->get_output_section_offset(folded.second);
-              gold_assert(secoffset != invalid_address);
-            }
 
 	  if (os == NULL)
 	    {
@@ -1653,7 +1582,7 @@ Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
               // so we leave the input value unchanged here.
 	      continue;
 	    }
-	  else if (secoffset == invalid_address)
+	  else if (out_offsets[shndx] == invalid_address)
 	    {
 	      uint64_t start;
 
@@ -1695,11 +1624,11 @@ Sized_relobj<size, big_endian>::do_finalize_local_symbols(unsigned int index,
 	    }
           else if (lv.is_tls_symbol())
 	    lv.set_output_value(os->tls_offset()
-				+ secoffset
+				+ out_offsets[shndx]
 				+ lv.input_value());
 	  else
 	    lv.set_output_value((relocatable ? 0 : os->address())
-				+ secoffset
+				+ out_offsets[shndx]
 				+ lv.input_value());
 	}
 
@@ -1744,42 +1673,6 @@ Sized_relobj<size, big_endian>::do_set_local_dynsym_offset(off_t off)
   return this->output_local_dynsym_count_;
 }
 
-// If Symbols_data is not NULL get the section flags from here otherwise
-// get it from the file.
-
-template<int size, bool big_endian>
-uint64_t
-Sized_relobj<size, big_endian>::do_section_flags(unsigned int shndx)
-{
-  Symbols_data* sd = this->get_symbols_data();
-  if (sd != NULL)
-    {
-      const unsigned char* pshdrs = sd->section_headers_data
-                                    + This::shdr_size * shndx;
-      typename This::Shdr shdr(pshdrs);
-      return shdr.get_sh_flags(); 
-    }
-  // If sd is NULL, read the section header from the file.
-  return this->elf_file_.section_flags(shndx); 
-}
-
-// Get the section's ent size from Symbols_data.  Called by get_section_contents
-// in icf.cc
-
-template<int size, bool big_endian>
-uint64_t
-Sized_relobj<size, big_endian>::do_section_entsize(unsigned int shndx)
-{
-  Symbols_data* sd = this->get_symbols_data();
-  gold_assert (sd != NULL);
-
-  const unsigned char* pshdrs = sd->section_headers_data
-                                + This::shdr_size * shndx;
-  typename This::Shdr shdr(pshdrs);
-  return shdr.get_sh_entsize(); 
-}
-
-
 // Write out the local symbols.
 
 template<int size, bool big_endian>
@@ -2011,19 +1904,15 @@ Sized_relobj<size, big_endian>::map_to_kept_section(
     unsigned int shndx,
     bool* found) const
 {
-  Relobj* kept_object;
-  unsigned int kept_shndx;
-  if (this->get_kept_comdat_section(shndx, &kept_object, &kept_shndx))
+  Kept_comdat_section *kept = this->get_kept_comdat_section(shndx);
+  if (kept != NULL)
     {
-      Sized_relobj<size, big_endian>* kept_relobj =
-	static_cast<Sized_relobj<size, big_endian>*>(kept_object);
-      Output_section* os = kept_relobj->output_section(kept_shndx);
-      Address offset = kept_relobj->get_output_section_offset(kept_shndx);
+      gold_assert(kept->object_ != NULL);
+      *found = true;
+      Output_section* os = kept->object_->output_section(kept->shndx_);
+      Address offset = kept->object_->get_output_section_offset(kept->shndx_);
       if (os != NULL && offset != invalid_address)
-	{
-	  *found = true;
-	  return os->address() + offset;
-	}
+        return os->address() + offset;
     }
   *found = false;
   return 0;
@@ -2221,14 +2110,27 @@ Object*
 make_elf_sized_object(const std::string& name, Input_file* input_file,
 		      off_t offset, const elfcpp::Ehdr<size, big_endian>& ehdr)
 {
-  Target* target = select_target(ehdr.get_e_machine(), size, big_endian,
-				 ehdr.get_e_ident()[elfcpp::EI_OSABI],
-				 ehdr.get_e_ident()[elfcpp::EI_ABIVERSION]);
-  if (target == NULL)
-    gold_fatal(_("%s: unsupported ELF machine number %d"),
-	       name.c_str(), ehdr.get_e_machine());
-  return target->make_elf_object<size, big_endian>(name, input_file, offset,
-						   ehdr);
+  int et = ehdr.get_e_type();
+  if (et == elfcpp::ET_REL)
+    {
+      Sized_relobj<size, big_endian>* obj =
+	new Sized_relobj<size, big_endian>(name, input_file, offset, ehdr);
+      obj->setup(ehdr);
+      return obj;
+    }
+  else if (et == elfcpp::ET_DYN)
+    {
+      Sized_dynobj<size, big_endian>* obj =
+	new Sized_dynobj<size, big_endian>(name, input_file, offset, ehdr);
+      obj->setup(ehdr);
+      return obj;
+    }
+  else
+    {
+      gold_error(_("%s: unsupported ELF file type %d"),
+		 name.c_str(), et);
+      return NULL;
+    }
 }
 
 } // End anonymous namespace.
