@@ -1,6 +1,6 @@
 // symtab.h -- the gold symbol table   -*- C++ -*-
 
-// Copyright 2006, 2007, 2008 Free Software Foundation, Inc.
+// Copyright 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 // Written by Ian Lance Taylor <iant@google.com>.
 
 // This file is part of gold.
@@ -27,6 +27,7 @@
 #include <utility>
 #include <vector>
 
+#include "gc.h"
 #include "elfcpp.h"
 #include "parameters.h"
 #include "stringpool.h"
@@ -56,6 +57,7 @@ class Output_section;
 class Output_segment;
 class Output_file;
 class Output_symtab_xindex;
+class Garbage_collection;
 
 // The base class of an entry in the symbol table.  The symbol table
 // can have a lot of entries, so we don't want this class to big.
@@ -208,6 +210,15 @@ class Symbol
   visibility() const
   { return this->visibility_; }
 
+  // Set the visibility.
+  void
+  set_visibility(elfcpp::STV visibility)
+  { this->visibility_ = visibility; }
+
+  // Override symbol visibility.
+  void
+  override_visibility(elfcpp::STV);
+
   // Return the non-visibility part of the st_other field.
   unsigned char
   nonvis() const
@@ -519,6 +530,10 @@ class Symbol
   bool
   needs_plt_entry() const
   {
+    // An undefined symbol from an executable does not need a PLT entry.
+    if (this->is_undefined() && !parameters->options().shared())
+      return false;
+
     return (!parameters->doing_static_link()
             && this->type() == elfcpp::STT_FUNC
             && (this->is_from_dynobj()
@@ -550,10 +565,10 @@ class Symbol
     if (parameters->doing_static_link())
       return false;
 
-    // A reference to a weak undefined symbol from an executable should be
+    // A reference to an undefined symbol from an executable should be
     // statically resolved to 0, and does not need a dynamic relocation.
     // This matches gnu ld behavior.
-    if (this->is_weak_undefined() && !parameters->options().shared())
+    if (this->is_undefined() && !parameters->options().shared())
       return false;
 
     // A reference to an absolute symbol does not need a dynamic relocation.
@@ -1140,6 +1155,28 @@ class Symbol_table
 
   ~Symbol_table();
 
+  void
+  set_gc(Garbage_collection* gc)
+  { this->gc_ = gc; }
+
+  Garbage_collection*
+  gc()
+  { return this->gc_; }
+
+  // During garbage collection, this keeps undefined symbols.
+  void
+  gc_mark_undef_symbols(); 
+
+  // During garbage collection, this ensures externally visible symbols
+  // are not treated as garbage while building shared objects.
+  void
+  gc_mark_symbol_for_shlib(Symbol* sym);
+
+  // During garbage collection, this keeps sections that correspond to 
+  // symbols seen in dynamic objects.
+  inline void
+  gc_mark_dyn_syms(Symbol* sym);
+
   // Add COUNT external symbols from the relocatable object RELOBJ to
   // the symbol table.  SYMS is the symbols, SYMNDX_OFFSET is the
   // offset in the symbol table of the first symbol, SYM_NAMES is
@@ -1309,7 +1346,7 @@ class Symbol_table
 
   // Write out the global symbols.
   void
-  write_globals(const Input_objects*, const Stringpool*, const Stringpool*,
+  write_globals(const Stringpool*, const Stringpool*,
 		Output_symtab_xindex*, Output_symtab_xindex*,
 		Output_file*) const;
 
@@ -1334,6 +1371,25 @@ class Symbol_table
   // The type of the list of common symbols.
   typedef std::vector<Symbol*> Commons_type;
 
+  // The type of the symbol hash table.
+
+  typedef std::pair<Stringpool::Key, Stringpool::Key> Symbol_table_key;
+
+  struct Symbol_table_hash
+  {
+    size_t
+    operator()(const Symbol_table_key&) const;
+  };
+
+  struct Symbol_table_eq
+  {
+    bool
+    operator()(const Symbol_table_key&, const Symbol_table_key&) const;
+  };
+
+  typedef Unordered_map<Symbol_table_key, Symbol*, Symbol_table_hash,
+			Symbol_table_eq> Symbol_table_type;
+
   // Make FROM a forwarder symbol to TO.
   void
   make_forwarder(Symbol* from, Symbol* to);
@@ -1347,6 +1403,12 @@ class Symbol_table
 		  unsigned int st_shndx, bool is_ordinary,
 		  unsigned int orig_st_shndx);
 
+  // Define a default symbol.
+  template<int size, bool big_endian>
+  void
+  define_default_version(Sized_symbol<size>*, bool,
+			 Symbol_table_type::iterator);
+
   // Resolve symbols.
   template<int size, bool big_endian>
   void
@@ -1360,7 +1422,8 @@ class Symbol_table
   void
   resolve(Sized_symbol<size>* to, const Sized_symbol<size>* from);
 
-  // Record that a symbol is forced to be local by a version script.
+  // Record that a symbol is forced to be local by a version script or
+  // by visibility.
   void
   force_local(Symbol*);
 
@@ -1401,7 +1464,8 @@ class Symbol_table
   template<int size, bool big_endian>
   Sized_symbol<size>*
   define_special_symbol(const char** pname, const char** pversion,
-			bool only_if_ref, Sized_symbol<size>** poldsym);
+			bool only_if_ref, Sized_symbol<size>** poldsym,
+			bool* resolve_oldsym);
 
   // Define a symbol in an Output_data, sized version.
   template<int size>
@@ -1475,9 +1539,9 @@ class Symbol_table
   // Write globals specialized for size and endianness.
   template<int size, bool big_endian>
   void
-  sized_write_globals(const Input_objects*, const Stringpool*,
-		      const Stringpool*, Output_symtab_xindex*,
-		      Output_symtab_xindex*, Output_file*) const;
+  sized_write_globals(const Stringpool*, const Stringpool*,
+		      Output_symtab_xindex*, Output_symtab_xindex*,
+		      Output_file*) const;
 
   // Write out a symbol to P.
   template<int size, bool big_endian>
@@ -1489,7 +1553,7 @@ class Symbol_table
 
   // Possibly warn about an undefined symbol from a dynamic object.
   void
-  warn_about_undefined_dynobj_symbol(const Input_objects*, Symbol*) const;
+  warn_about_undefined_dynobj_symbol(Symbol*) const;
 
   // Write out a section symbol, specialized for size and endianness.
   template<int size, bool big_endian>
@@ -1497,25 +1561,6 @@ class Symbol_table
   sized_write_section_symbol(const Output_section*, Output_symtab_xindex*,
 			     Output_file*, off_t) const;
 
-  // The type of the symbol hash table.
-
-  typedef std::pair<Stringpool::Key, Stringpool::Key> Symbol_table_key;
-
-  struct Symbol_table_hash
-  {
-    size_t
-    operator()(const Symbol_table_key&) const;
-  };
-
-  struct Symbol_table_eq
-  {
-    bool
-    operator()(const Symbol_table_key&, const Symbol_table_key&) const;
-  };
-
-  typedef Unordered_map<Symbol_table_key, Symbol*, Symbol_table_hash,
-			Symbol_table_eq> Symbol_table_type;
-
   // The type of the list of symbols which have been forced local.
   typedef std::vector<Symbol*> Forced_locals;
 
@@ -1602,6 +1647,7 @@ class Symbol_table
   Copied_symbol_dynobjs copied_symbol_dynobjs_;
   // Information parsed from the version script, if any.
   const Version_script_info& version_script_;
+  Garbage_collection* gc_;
 };
 
 // We inline get_sized_symbol for efficiency.
