@@ -1,6 +1,6 @@
 /* corefile.c
 
-   Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008, 2009
+   Copyright 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008
    Free Software Foundation, Inc.
 
    This file is part of GNU Binutils.
@@ -33,13 +33,13 @@ bfd *core_bfd;
 static int core_num_syms;
 static asymbol **core_syms;
 asection *core_text_sect;
-void * core_text_space;
+PTR core_text_space;
 
 static int min_insn_size;
 int offset_to_code;
 
 /* For mapping symbols to specific .o files during file ordering.  */
-struct function_map * symbol_map;
+struct function_map *symbol_map;
 unsigned int symbol_map_count;
 
 static void read_function_mappings (const char *);
@@ -61,23 +61,12 @@ parse_error (const char *filename)
   done (1);
 }
 
-/* Compare two function_map structs based on function name.
-   We want to sort in ascending order.  */
-
-static int
-cmp_symbol_map (const void * l, const void * r)
-{
-  return strcmp (((struct function_map *) l)->function_name, 
-		 ((struct function_map *) r)->function_name);
-}
-
 static void
 read_function_mappings (const char *filename)
 {
-  FILE * file = fopen (filename, "r");
+  FILE *file = fopen (filename, "r");
   char dummy[1024];
   int count = 0;
-  unsigned int i;
 
   if (!file)
     {
@@ -155,16 +144,11 @@ read_function_mappings (const char *filename)
 
   /* Record the size of the map table for future reference.  */
   symbol_map_count = count;
-
-  for (i = 0; i < symbol_map_count; ++i)
-    if (i == 0 || strcmp (symbol_map[i].file_name, symbol_map[i - 1].file_name))
-      symbol_map[i].is_first = 1;
-
-  qsort (symbol_map, symbol_map_count, sizeof (struct function_map), cmp_symbol_map);
 }
 
+
 void
-core_init (const char * aout_name)
+core_init (const char *aout_name)
 {
   int core_sym_bytes;
   asymbol *synthsyms;
@@ -450,121 +434,17 @@ get_src_info (bfd_vma addr, const char **filename, const char **name, int *line_
     }
 }
 
-/* Return number of symbols in a symbol-table file.  */
-
-static int 
-num_of_syms_in (FILE * f)
-{
-  const int BUFSIZE = 1024;
-  char * buf = (char *) xmalloc (BUFSIZE);
-  char * address = (char *) xmalloc (BUFSIZE);
-  char   type;
-  char * name = (char *) xmalloc (BUFSIZE);
-  int num = 0;
-  
-  while (!feof (f) && fgets (buf, BUFSIZE - 1, f))
-    {
-      if (sscanf (buf, "%s %c %s", address, &type, name) == 3)
-        if (type == 't' || type == 'T')
-          ++num;
-    }
-
-  free (buf);
-  free (address);
-  free (name);
-
-  return num;
-}
-
-/* Read symbol table from a file.  */
-
-void
-core_create_syms_from (const char * sym_table_file)
-{
-  const int BUFSIZE = 1024;
-  char * buf = (char *) xmalloc (BUFSIZE);
-  char * address = (char *) xmalloc (BUFSIZE);
-  char type;
-  char * name = (char *) xmalloc (BUFSIZE);
-  bfd_vma min_vma = ~(bfd_vma) 0;
-  bfd_vma max_vma = 0;
-  FILE * f;
-
-  f = fopen (sym_table_file, "r");
-  if (!f)
-    {
-      fprintf (stderr, _("%s: could not open %s.\n"), whoami, sym_table_file);
-      done (1);
-    }
-
-  /* Pass 1 - determine upper bound on number of function names.  */
-  symtab.len = num_of_syms_in (f);
-
-  if (symtab.len == 0)
-    {
-      fprintf (stderr, _("%s: file `%s' has no symbols\n"), whoami, sym_table_file);
-      done (1);
-    }
-
-  symtab.base = (Sym *) xmalloc (symtab.len * sizeof (Sym));
-
-  /* Pass 2 - create symbols.  */
-  symtab.limit = symtab.base;
-
-  if (fseek (f, 0, SEEK_SET) != 0)
-    {
-      perror (sym_table_file);
-      done (1);
-    }
-
-  while (!feof (f) && fgets (buf, sizeof (buf), f))
-    {
-      if (sscanf (buf, "%s %c %s", address, &type, name) == 3)
-        if (type != 't' && type != 'T')
-          continue;
-
-      sym_init (symtab.limit);
-
-      sscanf (address, "%" BFD_VMA_FMT "x", &(symtab.limit->addr) );
-
-      symtab.limit->name = (char *) xmalloc (strlen (name) + 1);
-      strcpy ((char *) symtab.limit->name, name);
-      symtab.limit->mapped = 0;
-      symtab.limit->is_func = TRUE;
-      symtab.limit->is_bb_head = TRUE;
-      symtab.limit->is_static = (type == 't');
-      min_vma = MIN (symtab.limit->addr, min_vma);
-      max_vma = MAX (symtab.limit->addr, max_vma);
-
-      ++symtab.limit;
-    }
-  fclose (f);
-
-  symtab.len = symtab.limit - symtab.base;
-  symtab_finalize (&symtab);
-
-  free (buf);
-  free (address);
-  free (name);
-}
-
-static int
-search_mapped_symbol (const void * l, const void * r)
-{
-    return strcmp ((const char *) l, ((const struct function_map *) r)->function_name);
-}
-
 /* Read in symbol table from core.
    One symbol per function is entered.  */
 
 void
-core_create_function_syms (void)
+core_create_function_syms ()
 {
-  bfd_vma min_vma = ~ (bfd_vma) 0;
+  bfd_vma min_vma = ~(bfd_vma) 0;
   bfd_vma max_vma = 0;
   int class;
-  long i;
-  struct function_map * found;
+  long i, found, skip;
+  unsigned int j;
 
   /* Pass 1 - determine upper bound on number of function names.  */
   symtab.len = 0;
@@ -574,12 +454,23 @@ core_create_function_syms (void)
       if (!core_sym_class (core_syms[i]))
 	continue;
 
-      /* Don't create a symtab entry for a function that has
+      /* This should be replaced with a binary search or hashed
+	 search.  Gross.
+
+	 Don't create a symtab entry for a function that has
 	 a mapping to a file, unless it's the first function
 	 in the file.  */
-      found = bsearch (core_syms[i]->name, symbol_map, symbol_map_count,
-		       sizeof (struct function_map), search_mapped_symbol);
-      if (found == NULL || found->is_first)
+      skip = 0;
+      for (j = 0; j < symbol_map_count; j++)
+	if (!strcmp (core_syms[i]->name, symbol_map[j].function_name))
+	  {
+	    if (j > 0 && ! strcmp (symbol_map [j].file_name,
+				   symbol_map [j - 1].file_name))
+	      skip = 1;
+	    break;
+	  }
+
+      if (!skip)
 	++symtab.len;
     }
 
@@ -609,9 +500,23 @@ core_create_function_syms (void)
 	  continue;
 	}
 
-      found = bsearch (core_syms[i]->name, symbol_map, symbol_map_count,
-		       sizeof (struct function_map), search_mapped_symbol);
-      if (found && ! found->is_first)
+      /* This should be replaced with a binary search or hashed
+	 search.  Gross.   */
+      skip = 0;
+      found = 0;
+
+      for (j = 0; j < symbol_map_count; j++)
+	if (!strcmp (core_syms[i]->name, symbol_map[j].function_name))
+	  {
+	    if (j > 0 && ! strcmp (symbol_map [j].file_name,
+				   symbol_map [j - 1].file_name))
+	      skip = 1;
+	    else
+	      found = j;
+	    break;
+	  }
+
+      if (skip)
 	continue;
 
       sym_init (symtab.limit);
@@ -622,9 +527,10 @@ core_create_function_syms (void)
       if (sym_sec)
 	symtab.limit->addr += bfd_get_section_vma (sym_sec->owner, sym_sec);
 
-      if (found)
+      if (symbol_map_count
+	  && !strcmp (core_syms[i]->name, symbol_map[found].function_name))
 	{
-	  symtab.limit->name = found->file_name;
+	  symtab.limit->name = symbol_map[found].file_name;
 	  symtab.limit->mapped = 1;
 	}
       else
@@ -635,11 +541,10 @@ core_create_function_syms (void)
 
       /* Lookup filename and line number, if we can.  */
       {
-	const char * filename;
-	const char * func_name;
+	const char *filename, *func_name;
 
-	if (get_src_info (symtab.limit->addr, & filename, & func_name,
-			  & symtab.limit->line_num))
+	if (get_src_info (symtab.limit->addr, &filename, &func_name,
+			  &symtab.limit->line_num))
 	  {
 	    symtab.limit->file = source_file_lookup_path (filename);
 
@@ -699,7 +604,7 @@ core_create_function_syms (void)
    One symbol per line of source code is entered.  */
 
 void
-core_create_line_syms (void)
+core_create_line_syms ()
 {
   char *prev_name, *prev_filename;
   unsigned int prev_name_len, prev_filename_len;
