@@ -56,7 +56,6 @@
 #define ARCH_maxq
 #define ARCH_mcore
 #define ARCH_mep
-#define ARCH_microblaze
 #define ARCH_mips
 #define ARCH_mmix
 #define ARCH_mn10200
@@ -198,7 +197,6 @@ disassembler (abfd)
 #endif
 #ifdef ARCH_i386
     case bfd_arch_i386:
-    case bfd_arch_l1om:
       disassemble = print_insn_i386;
       break;
 #endif
@@ -265,11 +263,6 @@ disassembler (abfd)
       disassemble = print_insn_mt;
       break;
 #endif
-#ifdef ARCH_microblaze
-    case bfd_arch_microblaze:
-      disassemble = print_insn_microblaze;
-      break;
-#endif
 #ifdef ARCH_msp430
     case bfd_arch_msp430:
       disassemble = print_insn_msp430;
